#!/usr/local/bin/php
<?

/*

sh4re - a secure peer to peer file sharing system
=================================================

Copyright (C) 2003 Vincent Negrier aka. sIX <six@aegis-corp.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

define("SH4_DEFAULT_CONF_FILE", ((strpos(strtoupper(PHP_OS), "WIN")!==false)?"C:\\sh4re\\":"/etc/sh4re/")."sh4re.conf");
define("SH4_INC_DIR", ((strpos(strtoupper(PHP_OS), "WIN")!==false)?"C:\\sh4re\\inc":"/usr/lib/sh4re/inc"));

include SH4_INC_DIR.DIRECTORY_SEPARATOR."sh4re-lib.php";

// Misc

define("SH4RED_HELP", <<<EOT
usage: ./sh4red.php [options]

-h --help             : this help page
-V --version          : show version number
-c --config=xx        : use specified config file
-a --address=##       : ip address to connect to
-p --port=##          : port to connect to
-k --key=secret       : use specified control key
-C --command="cmd"    : send command and exit
-q --quiet            : be quiet


EOT
);

// Help pages

define("HELP_PAGE_NONE", <<<EOT
help page not found

EOT
);


define("HELP_PAGE_DEFAULT", <<<EOT
available commands :
       
HELP         NETWORK      RIGHTS       NODEINFO     GRANT        REVOKE
SAY          MSG          FIND         LFIND        LIST         INFO
GET          HGET         CONNECT      DISCONNECT   STATUS       RESCAN
CANCEL       KILL         SHUTDOWN     QUIT         RECOMMEND    SET

usage: HELP [COMMAND]

EOT
);

define("HELP_PAGE_NETWORK", <<<EOT
show network informations including online nodes and total amount of sh4red
content on the network

usage: NETWORK

EOT
);

define("HELP_PAGE_RIGHTS", <<<EOT
show your rights and the one you granted on each node of the network

usage: RIGHTS

EOT
);

define("HELP_PAGE_NODEINFO", <<<EOT
get detailed informations about the specified node

usage: NODEINFO <NODEID>

EOT
);

define("HELP_PAGE_GRANT", <<<EOT
grant access to the specified node to your content

usage: GRANT <NODEID>

EOT
);

define("HELP_PAGE_REVOKE", <<<EOT
revoke access from the specified node to your content

usage: REVOKE <NODEID>

EOT
);

define("HELP_PAGE_SAY", <<<EOT
send a public text message to the network

usage: SAY <your text>

EOT
);

define("HELP_PAGE_MSG", <<<EOT
send a private text message to a network node

usage: MSG <NODEID> <your text>

EOT
);

define("HELP_PAGE_FIND", <<<EOT
find content on the network

usage: FIND <search-string>

the <search-string> is composed of space delimited tokens being either
<search-text> or <search-expression>s

<search-text> is either a word of text or a " or ' enclosed string that may
contain spaces

<search-expression> is a string of form <criterium><operator><value>[<suffix>]

<criterium> must be one of the following :
TYPE      AGE       SIZE      SAMEDIR   META

<operator> must be one of the following :
=         <         >

<value> and <suffix> depend on the criterium :

for TYPE, <value> must be "Audio", "Video" or "Other"
for AGE, <value> must be a number, and <suffix> must be one of the following:
d (for days), w (for weeks), m (for months), y (for years)
for SIZE, <value> must be a number, and <suffix> must be one of the following:
<none> (for bytes), k (for KBytes), m (for MBytes), g (for GBytes)
for SAMEDIR, <value> must be a valid file hash, or a "$" prefixed file id
for META, <value> must be set to "n" and meta-info search will be disabled

examples: FIND "man with no name" type=audio
          FIND age<10d
          FIND samedir=$1
          FIND futurama type=video size>100M
          FIND wiliam shatner rocketman

EOT
);

define("HELP_PAGE_LFIND", <<<EOT
find content on the local node (see HELP FIND for details)

usage: LFIND <search-string>

EOT
);

define("HELP_PAGE_LIST", <<<EOT
show results list from last search

usage: LIST

EOT
);

define("HELP_PAGE_INFO", <<<EOT
display extended informations including meta-info about the specified item

usage: INFO <FILEID>

EOT
);

define("HELP_PAGE_GET", <<<EOT
get file(s) from the network

usage: GET <files-list>

<files-list> is either a single <range>, or a comma serparated list of <range>

<range> can be of different forms :
"6" means file with ID = 6
"12-18" means files with ID ranging from 12 to 18 inclusive
"50-" means all files from the one with id 50 inclusive
"-10" means all files from 1 to the one with id 10 inclusive
"*" means all files in the previously acquired find list

examples: GET 3
          GET 3,4
          GET -8,10-
          GET 3-10
          GET 3,5-11,15,20-
          GET *

EOT
);

define("HELP_PAGE_HGET", <<<EOT
get file(s) from the network by hash

usage: HGET <hashes-list>

<hashes-list> is either a single hash, or a comma separated list of hashes

example: HGET fcef1cf509b5ba4324ab9eeb775122c0

EOT
);

define("HELP_PAGE_STATUS", <<<EOT
get current node status including pending file transfers and active control
connections

usage: STATUS

EOT
);

define("HELP_PAGE_CONNECT", <<<EOT
connect to the network

usage: CONNECT

EOT
);

define("HELP_PAGE_DISCONNECT", <<<EOT
disconnect from the network

usage: DISCONNECT

EOT
);

define("HELP_PAGE_RESCAN", <<<EOT
rescan network for download list availability

usage: RESCAN

EOT
);

define("HELP_PAGE_CANCEL", <<<EOT
cancel a download and erase incomplete file

usage: CANCEL <download-connection-id>

EOT
);

define("HELP_PAGE_KILL", <<<EOT
kill a connetion (download, upload, or control)

usage: KILL <any-connection-id>

EOT
);


define("HELP_PAGE_SHUTDOWN", <<<EOT
disconnect the node from the network and shutdown the daemon

usage: SHUTDOWN [reason text]

EOT
);

define ("HELP_PAGE_RECOMMEND", <<<EOT
recommends a file to all users
			      
usage : RECOMMEND <file-id>

<file-id> is a file id returned by LIST or FIND.

EOT
);			      

define ("HELP_PAGE_SET", <<<EOT
sets up various parameters

usage : SET [parameter [definition]]
			
invoke SET without argument to see a list of parameters.

EOT
);

define("COL_GREY","\033[0;37m");
define("COL_RED","\033[0;31m");
define("COL_GREEN","\033[0;32m");
define("COL_CYAN","\033[0;36m");
define("COL_LIGHT_BLACK","\033[1;30m");
define("COL_LIGHT_RED","\033[1;31m");
define("COL_LIGHT_GREEN","\033[1;32m");
define("COL_LIGHT_BROWN","\033[1;33m");
define("COL_LIGHT_BLUE","\033[1;34m");				   
define("COL_LIGHT_PURPLE","\033[1;35m");
define("COL_LIGHT_CYAN","\033[1;36m");				   
define("COL_WHITE","\033[1;37m");
define("COL_STD","\033[0m");			  

			 
// Display funcs
			    
			    
function net_info_s($net_info) {

	$tot = $net_info["network"]["network_name"]." : ".count($net_info["active_nodes"])."/".count($net_info["nodes"])." node(s) online/total";
	
	$ret .= nodes_list_s($net_info["nodes"], true, false, array_keys($net_info["active_nodes"]), $net_info["active_nodes"]);
	$ret .= "------------------------------------------------ ------------- -----------\n";
	$ret .= str_pad($tot, 49).str_pad(number_format($net_info["network"]["files_total"]), 12, " ", STR_PAD_LEFT)." ".str_pad(number_format($net_info["network"]["mbytes_total"]/1024, 2), 12, " ", STR_PAD_LEFT)."\n";

	return $ret;

}

function nodes_list_s($net_info_nodes, $r_list=false, $num_listed=false, $prefix_online=array(), $act_list=false) {

	ksort($net_info_nodes);
	
	if (is_array($r_list)) {

		$add_1 = " Local Flags  Remote Flags";
		$add_2 = " ------------ ------------";
	
	} else if ($r_list === true) {

		$add_1 = "        Files    GigaBytes";
		$add_2 = " ------------ ------------";
	
	}
	
	$ret  = "NodeID User     Node Name                       ".$add_1."\n";
	$lines .= COL_WHITE."-".COL_GREY."-".COL_LIGHT_BLACK."--".COL_GREY."-".COL_WHITE."- -".COL_GREY."-".COL_LIGHT_BLACK."----".COL_GREY."-".COL_WHITE."- -".COL_GREY."-".COL_LIGHT_BLACK."----------------------------".COL_GREY."-".COL_WHITE."-".$add_2."\n";
	$ret .= $lines;
	$nb_nodes = count($net_info_nodes);
	
	if ($net_info_nodes) foreach ($net_info_nodes as $k=>$node) {

		$ret .= in_array($node["node_id"], $prefix_online) ? COL_LIGHT_GREEN." *".COL_STD : COL_LIGHT_RED." *".COL_STD;
		$ret .= str_pad($node["node_id"], 4, " ", STR_PAD_BOTH)." ";
		$ret .= str_pad($node["node_short_name"], 8)." ";
		$ret .= str_pad($node["node_name"], 32);
		
		if (is_array($r_list)) {
		
//			$ha = @in_array($node["node_id"], $r_list["from"]);
//			$am = @in_array($node["node_id"], $r_list["to"]);
			
			$ret .= " ";
//			$ret .= str_pad($ha?COL_LIGHT_GREEN."Yes".COL_STD:COL_LIGHT_RED."No".COL_STD, 25, " ", STR_PAD_BOTH)." ";
//			$ret .= str_pad($am?COL_LIGHT_GREEN."Yes".COL_STD:COL_LIGHT_RED."No".COL_STD, 25, " ", STR_PAD_BOTH);
                        $ret .= str_pad($r_list["from"][$node["node_id"]]["node_rights"], 12, " ", STR_PAD_BOTH)." ";
			$ret .= str_pad($r_list["to"][$node["node_id"]]["node_rights"], 12, " ", STR_PAD_BOTH);
													    
		} else if ($r_list === true) {

			$n_files = $act_list[$k]["node_files_a"] + $act_list[$k]["node_files_v"] + $act_list[$k]["node_files_o"];
			$n_mbytes = (int) ($act_list[$k]["node_mbytes_a"] + $act_list[$k]["node_mbytes_v"] + $act_list[$k]["node_mbytes_o"]);

			$ret .= " ";

			if (($n_files == 0) && ($n_mbytes == 0)) {
				
				$n_files = $n_mbytes = "-";

			} else {

				$n_files = number_format($n_files);
				$n_mbytes = number_format($n_mbytes/1024, 2);
			
			}
			
			$ret .= str_pad($n_files, 12, " ", STR_PAD_LEFT)." ";
			$ret .= str_pad($n_mbytes, 12, " ", STR_PAD_LEFT);
		
		}

		$ret .= "\n";

	} else {

		$ret .= "not connected\n";
	
	}

	if ($num_listed) $ret .= "\n".$nb_nodes." node".($nb_nodes>1?"s":"")." listed\n";

	return $ret;

}

function echo_find_wait() {

	static $cnt;
	static $chr;

	if (!--$cnt) {
	
		$chr = ($chr + 1)%4;
		$chrs = array("|", "/" ,"-", "\\");
		$ch2 = array(COL_WHITE.".".COL_GREY.".".COL_LIGHT_BLACK.".".COL_STD,
			     COL_GREY.".".COL_WHITE.".".COL_GREY.".".COL_STD,
			     COL_LIGHT_BLACK.".".COL_GREY.".".COL_WHITE.".".COL_STD,
			     COL_GREY.".".COL_LIGHT_BLACK.".".COL_GREY.".".COL_STD);
		echo chr(13)."Searching ".$ch2[$chr].COL_LIGHT_BLACK." [".COL_LIGHT_BLUE.$chrs[$chr].COL_LIGHT_BLACK."]".COL_STD;

		$cnt = 64;
	
	}

}

function echo_find_end() {

	echo chr(13)."               ".chr(13);

}

function echo_find_result($r) {

	if ($r) {

		echo "FID# File Name                             Date                Size Type  Avail\n";
		echo "---- ------------------------------------- ---------- ------------- ----- -----\n";
		 
		foreach ($r as $k=>$f) {

			$l =  str_pad($k+1, 4, " ", STR_PAD_LEFT)." ";
			$l .= str_pad(substr($f["file_name"], 0, 37), 37)." ";
			$l .= substr($f["file_date"], 0, 10)." ";
			$l .= str_pad(number_format($f["file_size"]), 13, " ", STR_PAD_LEFT)." ";
			$l .= str_pad($f["file_type"], 5)." ";
			$l .= str_pad($f["avail"], 5, " ", STR_PAD_BOTH);

			echo $l."\n";

			++$n_files;
			$n_bytes += $f["file_size"];

		}

		echo "-------------------------------------------------------------------------------\n";

	}
	
	echo "Found ".sh4_size_format($n_bytes)." in ".(int)$n_files." file(s)\n";

}

function recommend_result($r,$num)
{
    global $slib;
    if ($r)
    {
	foreach ($r as $k=>$f) {
	    if ($k+1 == $num)
	    {
		echo ("recommending ".$f["file_name"]."\n");
		$slib->say("is recommending the ".$f["file_type"]." file named ".$f["file_name"]." : type 'hget ".$f["file_hash"]."' to get it.");
	    }
	}
    }
}
			      
function make_fancy_text($text,$col1,$col2)
{
    $result=ereg_replace("(\[|\]|\(|\))([0-9a-zA-Z])","\033COL_LIGHT_BLACK"."\\1"."\033COL_STD"."\\2".$col2,$text);
    $result=ereg_replace("( |^)([a-zA-Z0-9])","\\1".$col1."\\2".$col2,$result);
    $result=ereg_replace("([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*\:[0-9]*)",COL_LIGHT_CYAN."\\1".COL_STD,$result);
    $result=ereg_replace("\033COL_LIGHT_BLACK",COL_LIGHT_BLACK,$result);
    $result=ereg_replace("\033COL_STD",COL_STD,$result);
    $result=ereg_replace("$",COL_STD,$result);
    return $result;
}

function echo_head($title) {
    $max_size=80;
    $echo_title=make_fancy_text($title,COL_LIGHT_CYAN,COL_LIGHT_RED);
    $size=strlen($echo_title)-strlen($title)+32+$max_size;
    $str=str_pad(COL_WHITE."-".COL_GREY."-".COL_LIGHT_BLACK."[".COL_LIGHT_BLUE." ".make_fancy_text($title,COL_LIGHT_CYAN,COL_CYAN)." ".COL_LIGHT_BLACK."]",$size,"-");
    echo $str.COL_GREY."-".COL_WHITE."-".COL_STD."\n";
}
       
function echo_tail() {
    echo COL_WHITE."-".COL_GREY."-".COL_LIGHT_BLACK."---------------------------------------------------------------------------".COL_GREY."-".COL_WHITE."-".COL_STD."\n";
}
       
function echo_file_info($file) {

    echo_head("File Info");
    echo "Name : ".$file["file_name"]."\n";
    if ($file["file_directory"]) echo "Dir  : ".$file["file_directory"]."\n";
    echo "Date : ".$file["file_date"]."\n";
    echo "Size : ".number_format($file["file_size"])." bytes\n";
    echo "Hash : ".$file["file_hash"]."\n";
    echo "Type : ".$file["file_type"]."\n";
    echo "#Avl : available at ".$file["avail"]." node(s)\n";
    
    echo_tail();
    foreach (array_keys($file) as $k) if ($maxl < strlen($k)) $maxl = strlen($k);
    foreach ($file as $k=>$v) if ((substr($k, 0, 5) != "file_") && ($k != "avail")) echo str_pad($k, $maxl)." : ".$v."\n";
    echo_tail();
}


			      
function echo_status($status) {

//	echo COL_LIGHT_GREY."--[".COL_LIGHT_BLUE." Node Status ".COL_LIGHT_GREY."]----------------------------------------------------------------\033[1;37m\n";
    echo_head("Node Status");
    echo make_fancy_text("Network connection",COL_WHITE,COL_GREY)." : ".($status["master_cnx"] ? (COL_LIGHT_GREEN."Online".COL_STD." (".strtok($status["master_cnx"], "#").")") : COL_LIGHT_RED."Offline".COL_STD)."\n";
    echo make_fancy_text("Node uptime",COL_WHITE,COL_GREY)."        : ".ts_to_string($status["current_time"] - $status["start_time"])."\n";
    echo make_fancy_text("Xfer connections",COL_WHITE,COL_GREY)."   : ".number_format($status["xfer_cnx_total"])." (".number_format($status["xfer_cnx_errors"])." errors)\n";
    echo make_fancy_text("Xfer bytes in/out",COL_WHITE,COL_GREY)."  : ".sh4_size_format($status["xfer_rcv_bytes"])." / ".sh4_size_format($status["xfer_sent_bytes"])." (total ".sh4_size_format($status["xfer_rcv_bytes"] + $status["xfer_sent_bytes"]).")\n";
    
    if ($status["control"]) {
	
	//echo "-- Control Connections --------------------------------------------------------\n";
	echo_head("Control Connections");
	$cli = 0;
		
		foreach ($status["control"] as $k=>$ctl) {

			$GLOBALS["controls_index"][++$cli] = array($k);
			echo str_pad("C".$cli, 5)." ".str_pad($ctl["remote_ip"].":".$ctl["remote_port"], 21)." ".str_pad(date("M d H:i", $ctl["accept_time"]), 12)." (idle ".ts_to_string($ctl["idle_time"]).")\n";
		
		}
	
	}

	$GLOBALS["downloads_index"] = array();
    $tot_d=0;
    $tot_u=0;
    $f_u=0;
    $f_d=0;
    $h_u=0;
    $h_d=0;
	if ($status["downloads"]) {
	
		//$u_s = " Downloads ";
		
		//echo "--".str_pad($u_s, 77, "-")."\n";
	    echo_head("Downloads");
		$dli = 0;
		
	    foreach ($status["downloads"] as $k=>$dl) {
		
		$fn = $dl["file_name"]
		  or $fn = $k;
		
		if (strlen($fn) > 21) $fn = "+".substr($fn, -20);

		$db_s = $dl["blocks_25c"]
		  or $db_s="-------------------------";
		$db_s=ereg_replace("(\#*)(\|*)(\-*)$",COL_GREEN."\\1".COL_LIGHT_CYAN."\\2".COL_LIGHT_BLACK."\\3",$db_s);
		    //$db_s=ereg_replace("(|*)(.*$)","3"."\\1"."2"."\\2",$db_s);
		//$db_s=ereg_replace("([\-]*)","4"."\\1"."2",$db_s);
		    
		$GLOBALS["downloads_index"][++$dli] = $k;
		$GLOBALS["downloads_cnx_index"][$dli] = array();
		echo str_pad("D".$dli, 5)." ".($dl["connections"]?COL_LIGHT_CYAN:COL_LIGHT_RED).str_pad($fn, 21).COL_LIGHT_BLACK." (".COL_STD.str_pad(number_format($dl["file_size"]), 13, " ", STR_PAD_LEFT)." bytes".COL_LIGHT_BLACK.")   ".COL_LIGHT_BLACK."[".$db_s."]".COL_STD."\n";
		$f_d+=1;
		if ($dl["connections"]) foreach ($dl["connections"] as $ck=>$cnx) {
		    
		    $GLOBALS["downloads_cnx_index"][$dli][] = $ck;
		    echo str_pad("D".$dli, 5)." ".str_pad($cnx["node_name"], 21)." ".str_pad($cnx["ip_addr"].":".$cnx["ip_port"], 21)."   ".str_pad(ts_to_string(time() - $cnx["connect_time"]), 15)." ".str_pad((sh4_size_format($cnx["rate_in"])."/s"), 11, " ", STR_PAD_LEFT)."\n";
		    $tot_d+=$cnx["rate_in"];
		    $h_d+=1;
			}
		
		}

	}

	$GLOBALS["uploads_index"] = array();
	
	if ($status["uploads"]) {
	
		//$u_s = " Uploads (".$status["slots_up_used"]."/".$status["slots_up"].") ";
		echo_head("Uploads (".$status["slots_up_used"]."/".$status["slots_up"].")");
		//echo "--".str_pad($u_s, 77, "-")."\n";

		$uli = 0;
		
		foreach ($status["uploads"] as $k=>$ul) {

		    $fn = $ul["file_name"]
		      or $fn = $k;
		    
		    if (strlen($fn) > 21) $fn = "+".substr($fn, -20);
		    
		    $db_s = $ul["blocks_25c"]
		      or $db_s="-------------------------";
		    $db_s=ereg_replace("(\|*)(\-*)$",COL_LIGHT_CYAN."\\1".COL_LIGHT_BLACK."\\2",$db_s);
		    $GLOBALS["uploads_index"][++$uli] = array($k);
		    echo str_pad("U".$uli, 5)." ".str_pad($fn, 21)." (".str_pad(number_format($ul["file_size"]), 13, " ", STR_PAD_LEFT)." bytes)   ".COL_LIGHT_BLACK."[".$db_s."]".COL_STD."\n";
		    
		    echo str_pad("U".$uli, 5)." ".str_pad($ul["node_name"], 21)." ".str_pad($ul["ip_addr"].":".$ul["ip_port"], 21)."   ".str_pad(ts_to_string(time() - $ul["accept_time"]), 15)." ".str_pad((sh4_size_format($ul["rate_out"])."/s"), 11, " ", STR_PAD_LEFT)."\n";
		    $tot_u+=$ul["rate_out"];
		    $f_u+=1;
		    $h_u+=1;
		}
	
	}
    
	
    //	echo "-------------------------------------------------------------------------------\n";
    echo_head("Totals");
    echo "UP   ".COL_WHITE."-".COL_GREY."-".COL_LIGHT_BLACK."> ".COL_LIGHT_RED.$f_u.COL_STD." connexion".($f_u>1?"s":"").", ".COL_LIGHT_RED.sh4_size_format($tot_u).COL_STD."/s \n";
    echo "DOWN ".COL_WHITE."-".COL_GREY."-".COL_LIGHT_BLACK."> ".COL_LIGHT_GREEN.$f_d.COL_STD." file".($f_d>1?"s":"").", ".COL_LIGHT_GREEN.$h_d.COL_STD." host",($h_d>1?"s":"").", ".COL_LIGHT_GREEN.sh4_size_format($tot_d).COL_STD."/s\n";
    echo_tail();
}

function echo_net_msg($msg_type,$msg)
{
    global $kid;
    global $prompt;
    global $act_n;
    
    switch($msg_type)
    {
     case S2P_PKT_MASTER_NOTICE:
	break;
     case S2P_PKT_MASTER_FATAL:
	break;
     case S2P_PKT_NOTICE:
	echo ("\n".make_fancy_text("[".date("H:i")."]",COL_WHITE,COL_GREY)." ".COL_LIGHT_CYAN."* ".make_fancy_text($msg,COL_WHITE,COL_GREY).COL_STD."\n");
	break;
     case S2P_PKT_PUBMSG:
	$u = sh4_unserialize($msg);
	echo ("\n".make_fancy_text("[".date("H:i")."] ".$u["name"]."> ".$u["msg"],COL_WHITE,COL_GREY)."\n");
	break;
     case S2P_PKT_PRIVMSG:
	$u = sh4_unserialize($msg);
	echo ("\n".make_fancy_text("[".date("H:i")."] MSG:".$act_n["active_nodes"][$u["node_id"]]["node_short_name"]."@".$u["name"]."(".$u["node_id"].")> ".$u["msg"],COL_WHITE,COL_GREY)."\n");
	break;
     default:
    }
    echo "$prompt";
    //posix_kill($kid,SIGURG);
}
			      
function node_info_s($an) {

	//$ret = "-- Node MOTD ------------------------------------------------------------------\n";
    echo_head("Node MOTD");
    echo $an["node_motd"]."\n";
    //"-- Node Info ------------------------------------------------------------------\n";
    echo_head("Node Info");
    echo "ID#    : ".($an["node_netop"] ? "@" : "").$an["node_id"]."\n";
    echo "User   : ".$an["node_short_name"]."\n";
    echo "Name   : ".$an["node_name"]."\n";
    echo "URL    : ".COL_LIGHT_CYAN.$an["node_url"].COL_STD."\n";
    echo "Agent  : ".$an["s2p_agent"]."\n";
    echo "Files  : ".number_format($an["node_files_a"] + $an["node_files_v"] + $an["node_files_o"])." (V:".number_format($an["node_files_v"])." A:".number_format($an["node_files_a"])." O:".number_format($an["node_files_o"]).")\n";
    echo "MBytes : ".number_format($an["node_mbytes_a"] + $an["node_mbytes_v"] + $an["node_mbytes_o"])." (V:".number_format($an["node_mbytes_v"])." A:".number_format($an["node_mbytes_a"])." O:".number_format($an["node_mbytes_o"]).")\n";
     //"-------------------------------------------------------------------------------\n";
    echo_tail();
	return $ret;
}

function do_settings ($set) {
    global $settings;
    
    echo_head("Settings");
    foreach ($settings as $setting => $value)
    {
	if ($setting == "repository")
	  foreach ($value as $dir)
	  {
	      echo make_fancy_text(str_pad($setting,30),COL_WHITE,COL_GREY).COL_LIGHT_CYAN."$dir".COL_STD."\n";
	      $setting="";
	  }
	else
	  echo make_fancy_text(str_pad($setting,30),COL_WHITE,COL_GREY).COL_LIGHT_CYAN."$value".COL_STD."\n";
    }
    echo_tail();
}
			      
function parse_get_arg_to_hashes($s, $result) {

	$hashes = array();
	
	foreach (sh4_parse_range($s, count($result) - 1) as $a) if ((is_array($result[$a - 1])) && (!in_array($result[$a - 1]["file_hash"], $hashes))) $hashes[$result[$a - 1]["file_hash"]] = $result[$a - 1]["file_name"];
	
	return $hashes;

}
			      
function ts_to_string($timestamp) {
				  
    $up_d=(int)($timestamp/86400);
    $up_h=((int)($timestamp/3600))%24;
    $up_m=((int)($timestamp/60))%60;
    $up_s=$timestamp%60;
    
    $tstr=($up_d?($up_d."d, "):"").(($up_h||$up_d)?($up_h."h, "):"").(($up_m||$up_h||$up_d)?($up_m."m, "):"").$up_s."s";
    
    return ($tstr);
    
}
			      
function repaint_commandline($signo)
{
    global $prompt;
    echo "passed $signo\n";
    print "-->$prompt\n";
    flush();
}
// Init

$cmdline = sh4_parse_cmdline_args(array(	"-h" => "--help",
											"-V" => "--version",
											"-c" => "--config",
											"-a" => "--address",
											"-p" => "--port",
											"-k" => "--key",
											"-C" => "--command",
											"-q" => "--quiet"));

if ($cmdline["-V"]) die(SH4_VERSION."\n");
if ($cmdline["-h"]) die(SH4RED_HELP);

$host = $cmdline["-a"] or
$host = "127.0.0.1";

$port = $cmdline["-p"] or
$port = 10102;

$sh_cmd = $cmdline["-C"];

			      
if (!$cmdline["-q"]) echo "sh4re-cli/".SH4_VERSION." (c) 2003 by sIX / aEGiS <six@aegis-corp.org>\n";
if (!$cmdline["-q"]) echo "sh4re-cli/".SH4_VERSION." modified by JmB / aEGiS <jimbo@aegis-corp.org>\n";
			      
$conf_f = $cmdline["-c"]
or $conf_f = SH4_DEFAULT_CONF_FILE;

if ($conf = sh4_parse_config($conf_f)) {
  $ctl_key = $conf["controlkey"][0];
} else if ($cmdline["-c"]) {
  errexit("FATAL: unable to read conf file '$conf_f'");
}

$settings = Array (
    "availscaninterval" =>
		   ($conf["availscaninterval"][0]?$conf["availscaninterval"][0]:SH4_DEFAULT_DLIST_ASK_INTERVAL),
    "availscantimeout" =>
		   ($conf["availscantimeout"][0]?$conf["availscantimeout"][0]:SH4_DEFAULT_ASK_TIMEOUT),
    "configfile" => $conf_f,
    "downloaddir"=> trim($conf["downloaddir"][0]),
    "repository" => $conf["repository"],
    "workingdir" => trim($conf["workingdir"][0]),
    "xferlistenertimeout" =>
		   ($conf["xferlistenertimeout"][0]?$conf["xferlistenertimeout"][0]:SH4_DEFAULT_XFER_LISTENER_TIMEOUT),
    "xferports"  => trim($conf["xferports"][0])
		   );
			      
			      
if ($cmdline["-k"]) $ctl_key = $cmdline["-k"];

$slib = new sh4re_lib($host, $port, $ctl_key);

$slib->s2p_agent = "sh4re-cli/".SH4_VERSION;

if (($slib->connect()) && ($node = $slib->get_node_info())) {
    if (!$cmdline["-q"]) echo "connected to ".$node["node_name"]." at $host:$port (".$slib->cnx->remote_s2p_agent.")\n";
} else {
    die("unable to connect to $host:$port\n");
}

/*			      
if ($act_n = $slib->get_net_info(SH4_NI_ACTIVE_NODES)) {
    for ($i =1; $act_n["active_nodes"][$i]["node_name"] != $node["node_name"]; $i++);
}
*/
$act_n = $slib->get_net_info(SH4_NI_ACTIVE_NODES);
			      
if (!$cmdline["-q"]) echo "You are authenticated as ".$act_n["active_nodes"][$node["my_node_id"]]["node_short_name"].", your Node ID is #".$node["my_node_id"]."\n"; 
$prompt=COL_LIGHT_BLACK.$act_n["active_nodes"][$node["my_node_id"]]["node_short_name"].COL_LIGHT_BLUE."@".COL_LIGHT_BLACK.$node["node_name"].COL_LIGHT_BLUE."> ".COL_STD;
$sockets=array();
socket_create_pair(AF_UNIX,SOCK_STREAM,0,$sockets);			      
//foreach ($i=readline_info() as $value) echo "-$value-\n";
declare(ticks=1);
$net=1;

if ($kid=pcntl_fork())
  {
      if ($sh_cmd)
	$exit=1;
      $capture=1;
  }else{
      pcntl_signal(SIGURG,"repaint_commandline",0);
  //    readline_completion_function("repaint_commandline");
      while( !$exit )
      {
	  if ($sh_cmd)
	  {
	      $s=$sh_cmd;
	      $exit=1;
	  }
	  else
	  {
	      declare (ticks=1) {
		  $s=readline($prompt);}
	      readline_add_history($s);
	  }
	  if ($s) {
	      $s = rtrim($s, "\n\r");		
	      $cmd = trim(strtoupper(strtok($s, " ")));
	      $arg = strtok("");
	      socket_write($sockets[1],"NET_STOP");
	      switch ($cmd) {
		  
	       case "HELP":
		  if (!$arg2 = trim($arg)) $arg2 = "DEFAULT";
		  if (strtoupper($arg2) == "HELP") $arg2 = "DEFAULT";
		  if (!defined($cn = ("HELP_PAGE_".strtoupper(trim($arg2))))) $cn = "HELP_PAGE_NONE";
		  echo constant($cn);
		  break;
		  
	       case "":
		  $slib->shadok->run(-1);
		  break;
		  
	       case "NETWORK":
		  if ($net_info = $slib->get_net_info()) echo net_info_s($net_info)."\n";
		  break;
		  
	       case "NODEINFO":
		  if ($act_n = $slib->get_net_info(SH4_NI_ACTIVE_NODES)) {
		      if (!in_array($arg, array_keys($act_n["active_nodes"]))) {
			  echo "node $arg is not online\n";	
		      }else{
			  node_info_s($act_n["active_nodes"][$arg]);
		      }
		  }
		  break;
			  
	       case "RIGHTS":
		  if (($net_info = $slib->get_net_info(SH4_NI_NODES | SH4_NI_ACTIVE_NODES)) && is_array($r_list = $slib->get_rights_list())) echo nodes_list_s($net_info["nodes"], $r_list, true, array_keys($net_info["active_nodes"]))."\n";
		  break;
		  
	       case "GRANT":
		  if ($slib->access_grant($arg)) {
		      echo "granted $arg\n";
		  } else {
		      echo "grant $arg failed\n";
		  }
		  break;
		  
	       case "REVOKE":
		  if ($slib->access_revoke($arg)) {
		      echo "revoked $arg\n";
		  } else {
		      echo "revoke $arg failed\n";
		  }
		  break;
			  
	       case "SAY":
		  $slib->say($arg);
		  break;
	       
	       case "RECOMMEND":
		  recommend_result($find_result,$arg);
		  break;
		  
	       case "MSG":
		  $nto = strtok($arg, " ");
		  $msg = strtok("");
		  $slib->say_to($nto, $msg);
		  echo COL_GREY."Sa".COL_WHITE."i".COL_GREY."d ".COL_WHITE."'$msg'".COL_GREY." to node ".COL_WHITE."#$nto".COL_STD."\n";
		  break;
		  
	       case "LFIND":
	       case "FIND":
		  if (strpos($arg, '$') !== false) {
		      $r = array();
		      if ($find_result) foreach ($find_result as $k=>$v) $r['$'.($k + 1)] = $v["file_hash"];
		      $arg = strtr($arg, $r);
		      unset($r);
		  }
		  $find_result = $slib->find_files($arg, ($cmd == "LFIND"), ($cmdline["-q"] ? false : "echo_find_wait"));
		  if (!$cmdline["-q"]) echo_find_end();
		  echo_find_result($find_result);
			  break;
		  
	       case "LIST":
		  echo_find_result($find_result);
		  break;
			  
	       case "INFO":
		  if ($f = $find_result[$arg - 1]) {
			      echo_file_info($f);
		  } else {
		      echo "item not found\n";
		  }
			  break;
		  
	       case "GET":
		  if ($hlist = parse_get_arg_to_hashes($arg, $find_result)) {
		      $slib->get_files($hlist);
			  }
		  break;
		  
	       case "HGET":
		  if (($arg = trim($arg)) && (strlen($arg) == SH4_FILE_HASH_LEN))
		  {
		      $hlist=array($arg => null);
		      $slib->get_files($hlist);
		  }
		  else echo make_fancy_text("Something in your hash was wrong .... HGET failed.",COL_LIGHT_RED,COL_RED)."\n";
		  break;
		  
	       case "STATUS":
		  echo_status($slib->get_node_status());
		  break;
		  
	       case "QUIT":
	       case "EXIT":
		  $exit = true;
		  socket_write($sockets[1],"NET_QUIT");
		  echo "So long... and thanks for all the fish.\n";
		  break;
		  
	       case "SHUTDOWN":
		  if ($arg) $msg = $arg;
		  else $msg = "requested";
		  echo "shutdown in progress\n";
		  $slib->shutdown($msg);
		  break;
		  
	       case "CONNECT":
		  $slib->net_connect();
		  break;
		  
	       case "DISCONNECT":
		  $slib->net_disconnect();
		  break;
		  
	       case "RESCAN":
		  $slib->rescan();
		  break;
	       
	       case "SET":
		  do_settings($arg);
		  break;
		  
	       case "CANCEL":
		  if (strtoupper($arg{0}) !== "D") {
		      echo "can only cancel downloads\n";
		      break;
		  }
		  $dl_id = (int)substr($arg, 1);
		  if (!in_array($dl_id, array_keys($GLOBALS["downloads_index"]))) {
		      echo "not an active download\n";
		      break;
		  }
		  $slib->cancel_download($GLOBALS["downloads_index"][$dl_id]);
		  break;
		  
	       case "KILL":
		  $kill_t = strtoupper($arg{0});
		  $kill_id = (int)substr($arg, 1);
		  $kill_ids = array();
		  
		  switch ($kill_t) {
		      
		   case "D":
		      $kill_ids = $GLOBALS["downloads_cnx_index"][$kill_id];
		      break;
		      
		   case "U":
		      $kill_ids = $GLOBALS["uploads_index"][$kill_id];
		      break;
		      
		   case "C":
		      $kill_ids = $GLOBALS["controls_index"][$kill_id];
		      break;
		      
		   default:
		      echo "ask someone else for the job\n";
		      break 2;
		      
		  }
		  
		  if ($kill_ids) $slib->kill_sessions($kill_ids);
		  break;
		  
	       default:
		  echo "$cmd is not a valid command, try 'help'\n";
		  break;
	      }
	      //	$capture=0;
	      socket_write($sockets[1],"NET_START\032".readline_info(line_buffer)."\031");
	  } // (endof) if ($s)
      }
      die ();
  }
			      
while (!$exit)
    {
	$s="";
	$kidsock=$sockets[0];
	$child=array($kidsock);
	if ($ns=socket_select($child, $write=NULL, $toto=NULL,1))
	  $s = socket_read($sockets[0],100);
	if (strstr($s, "NET_STOP"))
	  $net=0;
	if (strstr($s, "NET_START"))
	{
	    $cmd=ereg_replace(".*NET_START\031(.*)\032.*","\\1",$s);
//	    echo "cmd is $cmd\n";
	    $net=1;
	}
	if (strstr($s, "NET_QUIT"))
	  $exit=1;
	
	if ($net)
	{
	    if (!$slib->connected) {
		
		echo "fatal: lost connection to sh4red\n";
		$exit = true;
		
	    }
	    
	    // empty MASTER_NOTICE bucket
	    
	    if ($slib->wait_pkts[S2P_PKT_MASTER_NOTICE]) {
		
		foreach ($slib->wait_pkts[S2P_PKT_MASTER_NOTICE] as $msg) echo "\n[".date("H:i")."] * ".$msg."\n";
		
		$slib->wait_pkts[S2P_PKT_MASTER_NOTICE] = array();
		
	    }
    
	    // empty MASTER_FATAL bucket
	
	    if ($slib->wait_pkts[S2P_PKT_MASTER_FATAL]) {
		
		foreach ($slib->wait_pkts[S2P_PKT_MASTER_FATAL] as $msg) echo "\n[".date("H:i")."] !!! ".$msg."\n";
		
		$slib->wait_pkts[S2P_PKT_MASTER_FATAL] = array();
		
	    }
	    
	    // empty NOTICE bucket
	    
	    if ($slib->wait_pkts[S2P_PKT_NOTICE]) {
		
		foreach ($slib->wait_pkts[S2P_PKT_NOTICE] as $msg)
		  echo_net_msg(S2P_PKT_NOTICE,$msg);
	    $slib->wait_pkts[S2P_PKT_NOTICE] = array();
		
	    }
	
	    // empty PUBMSG bucket
	    if ($slib->wait_pkts[S2P_PKT_PUBMSG]) {
		foreach ($slib->wait_pkts[S2P_PKT_PUBMSG] as $msg)
		  echo_net_msg(S2P_PKT_PUBMSG,$msg);
		$slib->wait_pkts[S2P_PKT_PUBMSG] = array();
	    }
	    
	    // empty PRIVMSG bucket
	    if ($slib->wait_pkts[S2P_PKT_PRIVMSG]) {
	    
		foreach ($slib->wait_pkts[S2P_PKT_PRIVMSG] as $msg) 
		    echo_net_msg(S2P_PKT_PRIVMSG,$msg);
		$slib->wait_pkts[S2P_PKT_PRIVMSG] = array();
		
	    }
	    
	    $slib->shadok->run(-1);
	}
    }
posix_kill ($kid,SIGKILL);			      
?>
