#!/usr/local/bin/php
<?

/*

sh4re - private and secure peer to peer file sharing system
===========================================================

Copyright (C) 2003 Vincent Negrier aka. sIX <six@aegis-corp.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

define("SH4_DEFAULT_CONF_FILE", ((strpos(strtoupper(PHP_OS), "WIN")!==false)?"C:\\sh4re\\":"/etc/sh4re/")."sh4re-masterd.conf");
define("SH4_DEFAULT_SEARCH_TIMEOUT", 10);
define("SH4_DEFAULT_ASK_TIMEOUT", 30);

define("SH4_INC_DIR", ((strpos(strtoupper(PHP_OS), "WIN")!==false)?"C:\\sh4re\\inc":"/usr/lib/sh4re/inc"));

include SH4_INC_DIR.DIRECTORY_SEPARATOR."sh4re-core.php";

define("SH4RE_MASTERD_HELP", <<<EOT
usage: ./sh4re-masterd.php [options]

-h --help             : this help page
-V --version          : show version number
-c --config="/...."   : use specified config file
-d --start-daemon     : start and run in background

EOT
);

class sh4re_s2p_connection extends s2p_connection {

	function on_s2p_accept() {

		if (is_array($GLOBALS["active_nodes"][$this->remote_node["node_id"]])) {

			$msg = $this->remote_node["node_name"]." is already connected to the network";

			techo("* ".$msg);
			$this->s2p_write(S2P_PKT_MASTER_FATAL, $msg, false, true);

			return false;
		
		}
		
		$join_arr = array(
			
			"ip"			=> $this->remote_ip,
			"short_name"	=> $this->remote_node["node_short_name"],
			"name"			=> $this->remote_node["node_name"]
			
		);
		
		$join_msg = $join_arr["name"]." [".$join_arr["short_name"]."@".$join_arr["ip"]."] has joined the network (".$this->remote_s2p_agent.")";
		
		techo("* ".$join_msg);
		$this->s2p_broadcast(S2P_PKT_MASTER_NOTICE, $join_msg);

		$GLOBALS["active_nodes"][$this->remote_node["node_id"]] = array("connection" => &$this);
	
	}

	function on_s2p_fatal($err_msg) {

		techo("! $err_msg");
	
	}
	
	function on_disconnect() {

		if ($this->handshake == 1) {
			
			$part_arr = array(
				
				"ip"			=> $this->remote_ip,
				"short_name"	=> $this->remote_node["node_short_name"],
				"name"			=> $this->remote_node["node_name"]
				
			);
			
			$part_msg = $part_arr["name"]." [".$part_arr["short_name"]."@".$part_arr["ip"]."] has left the network";

			techo("* ".$part_msg);
			$this->s2p_broadcast(S2P_PKT_MASTER_NOTICE, $part_msg);

			foreach ($GLOBALS["active_search"] as $k=>$as) if (($z = array_search($this->remote_node["node_id"], $as["nodes"])) !== false) unset ($GLOBALS["active_search"][$k]["nodes"][$z]);

			if ($this->remote_node["node_id"]) unset($GLOBALS["active_nodes"][$this->remote_node["node_id"]]);
		
			foreach ($GLOBALS["sk"]->active_sessions as $k=>$o) {
				
				if ($k != $this->conn_id) {

					$GLOBALS["sk"]->active_sessions[$k]->send_net_info(SH4_NI_NETWORK | SH4_NI_DISCONNECT, $this->remote_node["node_id"]);

				}

			}
			
		} else {

			techo("? aborted connection from ".$this->remote_ip);
		
		}
	
	}

	function on_s2p_packet($type, $msg) {

		switch ($type) {

			case S2P_PKT_INFO_SRV:

				techo("@ received node info packet from ".$this->remote_node["node_short_name"]);

				$info = sh4_unserialize($msg);

				if ($info["node_name"]) $this->remote_node["node_name"] = $info["node_name"];
				if ($info["node_url"]) $this->remote_node["node_url"] = $info["node_url"];
				if ($info["node_motd"]) $this->remote_node["node_motd"] = $info["node_motd"];

				if ($info["node_files_a"]) $this->remote_node["node_files_a"] = $info["node_files_a"];
				if ($info["node_files_v"]) $this->remote_node["node_files_v"] = $info["node_files_v"];
				if ($info["node_files_o"]) $this->remote_node["node_files_o"] = $info["node_files_o"];
				if ($info["node_mbytes_a"]) $this->remote_node["node_mbytes_a"] = $info["node_mbytes_a"];
				if ($info["node_mbytes_v"]) $this->remote_node["node_mbytes_v"] = $info["node_mbytes_v"];
				if ($info["node_mbytes_o"]) $this->remote_node["node_mbytes_o"] = $info["node_mbytes_o"];

				sh4_put_node($this->remote_node);
				
				$GLOBALS["active_nodes"][$this->remote_node["node_id"]]["node_info"] = $this->remote_node;

				foreach ($GLOBALS["sk"]->active_sessions as $k=>$o) {
					
					if ($k == $this->conn_id) {

						$GLOBALS["sk"]->active_sessions[$k]->send_net_info(SH4_NI_ALL);
					
					} else {

						$GLOBALS["sk"]->active_sessions[$k]->send_net_info(SH4_NI_NETWORK | SH4_NI_NODE_UPDATE, $this->remote_node);

					}

				}

			break;
			
			case S2P_PKT_SHUTDOWN:

			if ($this->remote_node["node_netop"]) {
			
				$msg = "network master server is shutting down (requested by ".$this->remote_node["node_name"].")";

				techo($msg);
				$this->s2p_broadcast(S2P_PKT_MASTER_NOTICE, $msg, true, true);

				$GLOBALS["shutdown"] = true;
			
			}
			
			break;
			
			case S2P_PKT_PUBMSG:

			techo("* ".$this->remote_node["node_name"]."> ".$msg);
			
			$bmsg = sh4_serialize(array("node_id" => $this->remote_node["node_id"], "name" => $this->remote_node["node_name"], "msg" => $msg));
			$this->s2p_broadcast(S2P_PKT_PUBMSG, $bmsg);
			
			break;

			case S2P_PKT_PRIVMSG:

			$u = sh4_unserialize($msg);

			if (is_array($GLOBALS["active_nodes"][$u["node_id"]])) {
			
				$bmsg = sh4_serialize(array("node_id" => $this->remote_node["node_id"], "name" => $this->remote_node["node_name"], "msg" => $u["msg"]));
				$GLOBALS["active_nodes"][$u["node_id"]]["connection"]->s2p_write(S2P_PKT_PRIVMSG, $bmsg);
			
			}
			
			break;
		
			case S2P_PKT_RIGHT_LIST:

			techo("@ received S2P_PKT_RIGHT_LIST from ".$this->remote_node["node_short_name"]);
			$this->s2p_write(S2P_PKT_RIGHT_LIST, sh4_serialize(rights_list($this->remote_node["node_id"])));
			
			break;

			case S2P_PKT_RIGHT_SET:
			
			$rmsg = sh4_unserialize($msg);
			
			if ($rnode = sh4_get_node_by_id($rmsg["node_id"])) {
			
				techo("@ received S2P_PKT_RIGHT_SET from ".$this->remote_node["node_short_name"]." (".($rmsg["right"]?"grant":"revoke")." ".$rnode["node_short_name"].")");
				sh4_right_set($rnode["node_id"], $this->remote_node["node_id"], $rmsg["right"]);
				
			} else {

				techo("@ received bogus S2P_PKT_RIGHT_SET from ".$this->remote_node["node_short_name"]." (bad node id ".$rmsg["node_id"].")");
			
			}

			if (is_array($GLOBALS["active_nodes"][$rmsg["node_id"]])) {

				$GLOBALS["active_nodes"][$rmsg["node_id"]]["connection"]->send_net_info(SH4_NI_NETWORK | SH4_NI_ACTIVE_NODES);

			}

			break;

			case S2P_PKT_FIND:

			$u = sh4_unserialize($msg);
			
			if ($u["req"]) {

				// Request

				++$GLOBALS["sid"];
				
				$a = array("req" => true, "sid" => $GLOBALS["sid"], "search" => $u["search"], "node_id" => $this->remote_node["node_id"], "node_name" => $this->remote_node["node_name"]);
				$s = sh4_serialize($a);
				
				$rgh = rights_list($this->remote_node["node_id"], "s");

				techo("S ".$this->remote_node["node_name"]."> ".$u["search"]);
				
				if ($rgh["to"]) foreach ($rgh["to"] as $ask_node) if ((is_object($GLOBALS["active_nodes"][$ask_node]["connection"])) && ($ask_node != $this->remote_node["node_id"])) {
					
					$GLOBALS["active_nodes"][$ask_node]["connection"]->s2p_write(S2P_PKT_FIND, $s);
					$asked_nodes[] = $ask_node;

				}

				$GLOBALS["active_search"][$GLOBALS["sid"]] = array("node" => $this->remote_node["node_id"], "nodes" => $asked_nodes, "sid" => $GLOBALS["sid"], "start" => time(), "results" => array());

			} else {

				// Reply
				
				$u["result"] = sh4_unserialize(gzinflate($u["result"]));
				
				if (is_array($GLOBALS["active_search"][$u["sid"]])) {

					unset($GLOBALS["active_search"][$u["sid"]]["nodes"][array_search($this->remote_node["node_id"], $GLOBALS["active_search"][$u["sid"]]["nodes"])]);

					foreach ($u["result"] as $res) {
					
						$avail = $GLOBALS["active_search"][$u["sid"]]["results"][$res["file_hash"]]["avail"];
						$GLOBALS["active_search"][$u["sid"]]["results"][$res["file_hash"]] = array_merge($GLOBALS["active_search"][$u["sid"]]["results"][$res["file_hash"]], $res);
						$GLOBALS["active_search"][$u["sid"]]["results"][$res["file_hash"]]["avail"] = $avail + 1;

					}

					++$GLOBALS["active_search"][$u["sid"]]["nb_results"];
					
					if (!$GLOBALS["active_search"][$u["sid"]]["nodes"]) {

						techo("S ".(int)($GLOBALS["active_search"][$u["sid"]]["nb_results"])." resp node".(($GLOBALS["active_search"][$u["sid"]]["nb_results"]>1)?"s":"").", sending result set back : ".count($GLOBALS["active_search"][$u["sid"]]["results"])." items");
						
						results_postproc($GLOBALS["active_search"][$u["sid"]]["results"]);
						
						$s = gzdeflate(sh4_serialize(array_values($GLOBALS["active_search"][$u["sid"]]["results"])));
						
						if (is_object($GLOBALS["active_nodes"][$GLOBALS["active_search"][$u["sid"]]["node"]]["connection"])) {
							
							$GLOBALS["active_nodes"][$GLOBALS["active_search"][$u["sid"]]["node"]]["connection"]->s2p_write(S2P_PKT_FIND, $s);

						}
						
						unset($GLOBALS["active_search"][$u["sid"]]);
					
					}
				
				} else {

					techo("! discarded late search result packet from ".$this->remote_node["node_name"]);
				
				}

			}
			
			break;


			case S2P_PKT_ASK_HASH:

			$u = sh4_unserialize($msg);
			
			if (isset($u["file_hash"])) {

				// Request

				++$GLOBALS["aid"];
				
				$a = array("aid" => $GLOBALS["aid"], "file_hash" => $u["file_hash"], "node_id" => $this->remote_node["node_id"], "node_name" => $this->remote_node["node_name"], "ip_addr" => ($u["ip_addr"] ? $u["ip_addr"] : $this->remote_ip));
				$s = sh4_serialize($a);
				
				$rgh = rights_list($this->remote_node["node_id"], "d");

				techo("A ".$this->remote_node["node_name"]."> ".$u["file_hash"]);
				
				if (!is_array($u["exclude_nodes"])) $exclude_nodes = array();
				else $exclude_nodes = $u["exclude_nodes"];
				
				foreach ($rgh["to"] as $ask_node) if ((is_object($GLOBALS["active_nodes"][$ask_node]["connection"])) && (!in_array($ask_node, $exclude_nodes))) $GLOBALS["active_nodes"][$ask_node]["connection"]->s2p_write(S2P_PKT_ASK_HASH, $s);

				$GLOBALS["active_ask"][$GLOBALS["aid"]] = array("node" => $this->remote_node["node_id"], "node_aid" => $u["aid"], "start" => time());

			
			} else {

				// Reply

				if (is_array($GLOBALS["active_ask"][$u["aid"]])) {

					$r = $u;

					$r["aid"] = $GLOBALS["active_ask"][$u["aid"]]["node_aid"];
					$r["node_id"] = $this->remote_node["node_id"];
					$r["node_name"] = $this->remote_node["node_name"];

					if (!$r["ip_addr"]) $r["ip_addr"] = $this->remote_ip;

					if (is_object($GLOBALS["active_nodes"][$GLOBALS["active_ask"][$u["aid"]]["node"]]["connection"])) {
						
						$GLOBALS["active_nodes"][$GLOBALS["active_ask"][$u["aid"]]["node"]]["connection"]->s2p_write(S2P_PKT_ASK_HASH, sh4_serialize($r));

					}
				
				} else {

					techo("! discarded late ask result packet from ".$this->remote_node["node_name"]);

				}
				
			}

			break;


			case S2P_PKT_PUT_HASHES:

			$u = sh4_unserialize($msg);
			$rgh = rights_list($this->remote_node["node_id"], "u");

			$r_arr = array("node_id" => $this->remote_node["node_id"], "node_short_name" => $this->remote_node["node_short_name"], "node_name" => $this->remote_node["node_name"], "hashes" => $u["hashes"]);

			$s = sh4_serialize($r_arr);

			if (((int)$u["node_id"]) && (in_array($u["node_id"], $rgh["to"])) && ($u["node_id"] != $this->remote_node["node_id"]) && (is_object($GLOBALS["active_nodes"][$u["node_id"]]["connection"]))) {
			
				$GLOBALS["active_nodes"][$u["node_id"]]["connection"]->s2p_write(S2P_PKT_GET_HASHES, $s);

				techo("% relayed network put from ".$this->remote_node["node_short_name"]." to node ".$u["node_id"]);

			} else if (($u["node_id"] === "*") && ($this->remote_node["node_netop"])) {

				foreach ($rgh["to"] as $ask_node) if (($ask_node != $this->remote_node["node_id"]) && (is_object($GLOBALS["active_nodes"][$ask_node]["connection"]))) $GLOBALS["active_nodes"][$ask_node]["connection"]->s2p_write(S2P_PKT_GET_HASHES, $s);

				techo("% relayed global network put from ".$this->remote_node["node_short_name"]);
			
			}

			break;

		}
	
	}

	function send_net_info($args=SH4_NI_NETWORK, $opt=false) {

		if ($this->handshake) $this->s2p_write(S2P_PKT_INFO_NET, sh4_serialize(net_info($args, $this->remote_node["node_id"], $opt)));
	
	}

}

function sh4_right_set($node_from, $node_to, $rights) {

	if ($rights) {

		$vr = SH4_VALID_RIGHTS;
		$vrl = strlen(SH4_VALID_RIGHTS);
		$rights_s = "";

		for ($a = 0; $a < $vrl; $a++) if (strpos($rights, $vr{$a}) !== false) $rights_s .= $vr{$a};
		
		$sql = "replace into nodes_rights (from_node_id, to_node_id, node_rights) values ($node_from, $node_to, \"".$rights_s."\")";
	
	} else {

		$sql = "delete from nodes_rights where from_node_id = $node_from and to_node_id = $node_to";
	
	}

	return mysql_query($sql);

}

function rights_list($node_id, $right = false) {

	$access = array("to" => array(), "from" => array());
	
	if ($right !== false) $add_where = " and instr(node_rights, \"".addslashes($right)."\")";
	
	$q = mysql_query("select to_node_id, node_rights from nodes_rights where from_node_id = $node_id".$add_where);
	while ($r = mysql_fetch_assoc($q)) $access["to"][$r["to_node_id"]] = ($right === false ? $r : $r["to_node_id"]);
	mysql_free_result($q);

	$q = mysql_query("select from_node_id, node_rights from nodes_rights where to_node_id = $node_id".$add_where);
	while ($r = mysql_fetch_assoc($q)) $access["from"][$r["from_node_id"]] = ($right === false ? $r : $r["from_node_id"]);
	mysql_free_result($q);

	return($access);

}

function get_all_nodes() {

	$nodes = array();
	
	$q = mysql_query("select node_id, node_short_name, node_name from nodes order by node_id");
	while ($r = mysql_fetch_assoc($q)) $nodes[$r["node_id"]] = $r;
	mysql_free_result($q);

	return $nodes;

}

function net_info($flags=SH4_NI_NETWORK, $node_id, $opt=false) {

	$net_info = array();

	$rgh = rights_list($node_id, "d");
	$rgh["to"][] = $node_id;

	if ($flags & SH4_NI_NETWORK) {
	
		$f_a = $f_v = $f_o = $s_a = $s_v = $s_o = 0;
		
		foreach ($GLOBALS["active_nodes"] as $nid=>$an) if (in_array($nid, $rgh["to"])) {
			
			$f_a += $an["node_info"]["node_files_a"];
			$f_v += $an["node_info"]["node_files_v"];
			$f_o += $an["node_info"]["node_files_o"];
			$s_a += $an["node_info"]["node_mbytes_a"];
			$s_v += $an["node_info"]["node_mbytes_v"];
			$s_o += $an["node_info"]["node_mbytes_o"];
		
		}

		$f_t = $f_a + $f_v + $f_o;
		$s_t = $s_a + $s_v + $s_o;
		
		$net_info["network"]["files_audio"] = $f_a;
		$net_info["network"]["files_video"] = $f_v;
		$net_info["network"]["files_other"] = $f_o;
		$net_info["network"]["files_total"] = $f_t;
		$net_info["network"]["mbytes_audio"] = $s_a;
		$net_info["network"]["mbytes_video"] = $s_v;
		$net_info["network"]["mbytes_other"] = $s_o;
		$net_info["network"]["mbytes_total"] = $s_t;

	}

	if ($flags & SH4_NI_ACTIVE_NODES) {
	
		foreach ($GLOBALS["active_nodes"] as $k=>$an) {
			
			$aa = $an["node_info"];
			$aa["s2p_agent"] = $an["connection"]->remote_s2p_agent;
		
			if (!in_array($k, $rgh["to"])) {

				$aa["node_files_a"] = $aa["node_files_v"] = $aa["node_files_o"] = $aa["node_mbytes_a"] = $aa["node_mbytes_v"] = $aa["node_mbytes_o"] = 0;
			
			}
			
			$net_info["active_nodes"][$k] = $aa;

		}
		
	}

	if ($flags & SH4_NI_NODES) {

		$net_info["nodes"] = get_all_nodes();

	}
	
	if ($flags & SH4_NI_DISCONNECT) {

		$net_info["node_disconnect"] = $opt;
	
	}

	if ($flags & SH4_NI_NODE_UPDATE) {

		$opt["s2p_agent"] = $GLOBALS["active_nodes"][$opt["node_id"]]["connection"]->remote_s2p_agent;
		
		if (in_array($opt["node_id"], $rgh["to"])) {
		
			$net_info["node_update"] = $opt;

		} else {

			$aa = $opt;
			$aa["node_files_a"] = $aa["node_files_v"] = $aa["node_files_o"] = $aa["node_mbytes_a"] = $aa["node_mbytes_v"] = $aa["node_mbytes_o"] = 0;
			$net_info["node_update"] = $aa;
		
		}
	
	}
	
	return $net_info;

}

function results_postproc_compare($a, $b) {

	if ($a["avail"] < $b["avail"]) return 1;
	if ($a["avail"] > $b["avail"]) return -1;
	return 0;

}

function results_postproc(&$results) {

	uasort($results, "results_postproc_compare");
	if (($sl = $GLOBALS["conf"]["searchlimit"][0]) && (count($results) > $sl)) $results = array_slice($results, 0, $sl);

}

// Init

$cmdline = sh4_parse_cmdline_args(array(	"-h" => "--help",
											"-V" => "--version",
											"-c" => "--config",
											"-d" => "--start-daemon"));

if ($cmdline["-V"]) die(SH4_VERSION."\n");
if ($cmdline["-h"]) die(SH4RED_HELP);

techo("sh4re-masterd/".SH4_VERSION." (c) 2003 by sIX / aEGiS <six@t0x.net>");

$start_time = time();

if (version_compare(phpversion(), "4.3.0")<0) errexit("FATAL: sh4re-update needs PHP >= 4.3)");
if (!is_callable("openssl_x509_read")) errexit("FATAL: openssl extension not available");

$conf_f = $cmdline["-c"]
or $conf_f = SH4_DEFAULT_CONF_FILE;

if (!$conf = sh4_parse_config($conf_f)) errexit("FATAL: unable to read conf file '$conf_f'");

$s_timeout = $conf["searchtimeout"][0]
or $s_timeout = SH4_DEFAULT_SEARCH_TIMEOUT;

$a_timeout = $conf["asktimeout"][0]
or $a_timeout = SH4_DEFAULT_ASK_TIMEOUT;

$my_ca = array($conf["sslcert"][0]);
$my_cert_s = @file_get_contents($conf["sslcert"][0]);
$my_cert = @openssl_x509_read($my_cert_s);
@openssl_x509_export($my_cert, $my_cert_s);
$my_key_s = @file_get_contents($conf["sslkey"][0]);
$my_public_key = @openssl_pkey_get_public($my_cert_s);
$my_key = @openssl_pkey_get_private($my_key_s);
$net_key = $conf["networkkey"][0];
unset($my_key_s);

if (!$my_public_key) errexit("FATAL: unable to load SSL certificate from ".$conf["sslcert"][0]);
if (!$my_key) errexit("FATAL: unable to load SSL private key from ".$conf["sslkey"][0]);
if (!ssl_check_keys($my_key, $my_public_key)) errexit("FATAL: crypto test failed, certificate and private key don't match");

techo("SSL keys loaded, crypto test passed");

if ((!mysql_connect($conf["mysqlhost"][0], $conf["mysqluser"][0], $conf["mysqlpass"][0])) || (!mysql_select_db($conf["mysqldb"][0]))) errexit("FATAL: unable to connect to MySQL");

techo("connected to MySQL database");

// netlib init

if (!$lsn_s = trim($conf["listener"][0])) errexit("FATAL: unable to find listener parameter");

$sk = new sk_server;
techo($sk->sk_info());

$sk_options = array(	"s2p_agent"			=> "sh4re-masterd/".SH4_VERSION,
						"net_key"			=> $net_key,
						"use_rsa"			=> true,
						"my_key"			=> $my_key,
						"my_cert_s"			=> $my_cert_s,
						"check_client_cert"	=> true,
						"my_ca"				=> $my_ca);

if (!$sk->register_connection_handler("tcp://$lsn_s", "sh4re_s2p_connection", 20, $sk_options)) errexit("FATAL: listen failed (".$sk->error.")");

techo("listening for node connections at tcp://$lsn_s");

if ($cmdline["-d"]) {

	if (SK_Toolkit::Run_In_Background()) {

		techo("started daemon mode, running in background");

	} else {

		techo("unable to start daemon mode");
	
	}

}

$active_nodes = array();
$active_search = array();
$active_ask = array();

$shutdown = false;

// main loop

while (!$shutdown) {
	
	$sk->run(1);

	$t = time();
	
	foreach ($sk->active_sessions as $k=>$as) if ($as->s2p_errors) {
		
		foreach ($as->s2p_errors as $err) techo("[".$as->remote_ip.":".$as->remote_port."] $err");
		unset($sk->active_sessions[$k]->s2p_errors);

	}

	// Send noop packets every minute to keep connection alive

	if (($t_noop + 60) <= $t) {

		foreach ($sk->active_sessions as $as) if ($as->handshake) $as->s2p_write(S2P_PKT_WOOPWOOPWOOP);
		$t_noop = $t;

	}
	
	// Invalidate timed out searches
	
	foreach ($active_search as $k=>$as) if (($as["start"] + $s_timeout <= $t) || (!$as["nodes"])) {

		if ($as["nodes"]) {

			techo("! Node(s) did not respond to search in time : ".implode(", ", $as["nodes"]));
		
		}
		
		techo("S ".count($active_search[$k]["nb_results"])." resp. node(s), sending result set back : ".count($active_search[$k]["results"])." items");
		
		results_postproc($active_search[$k]["results"]);
		
		$s = gzdeflate(sh4_serialize(array_values($active_search[$k]["results"])));

		if (is_object($active_nodes[$as["node"]]["connection"])) {
		
			$active_nodes[$as["node"]]["connection"]->s2p_write(S2P_PKT_FIND, $s);

		}
		
		unset($active_search[$k]);

	}

	// Invalidate timed out avail scans
	
	foreach ($active_ask as $k=>$aa) if ($aa["start"] + $a_timeout <= $t) {
		
		unset($active_ask[$k]);

	}

}

while ($sk->active_writebuffers) $sk->run(1);

techo("sh4re-masterd stopped");

?>
