#!/usr/local/bin/php
<?

/*

sh4re - private and secure peer to peer file sharing system
===========================================================

Copyright (C) 2003 Vincent Negrier aka. sIX <six@aegis-corp.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

define("SH4_DEFAULT_CONF_FILE", ((strpos(strtoupper(PHP_OS), "WIN")!==false)?"C:\\sh4re\\":"/etc/sh4re/")."sh4re.conf");
define("SH4_INC_DIR", ((strpos(strtoupper(PHP_OS), "WIN")!==false)?"C:\\sh4re\\inc":"/usr/lib/sh4re/inc"));

include SH4_INC_DIR.DIRECTORY_SEPARATOR."sh4re-core.php";

define("SH4_DEFAULT_MASTER_PORT", 10101);
define("SH4_DEFAULT_XFER_LISTENER_TIMEOUT", 15);
define("SH4_DEFAULT_DLIST_ASK_INTERVAL", 900);
define("SH4_DEFAULT_ASK_TIMEOUT", 60);
define("SH4_DEFAULT_SEARCH_LIMIT", 128);
define("SH4_DEFAULT_XFER_BLOCK_SIZE", 262144);

define("SH4_RATE_SAMPLE_INTERVAL", 1);
define("SH4_RATE_AVG_INTERVAL", 5);

define("SH4RED_HELP", <<<EOT
usage: ./sh4red.php [options]

-h --help             : this help page
-V --version          : show version number
-n --new-network-cert : accept new network SSL certificate
-d --start-daemon     : start and run in background

EOT
);

// Network master connetion handler

class masterd_s2p_connection extends s2p_connection {

	function on_s2p_connect() {

		$conn_msg = "connected to ".$this->remote_cert_cn." (".$this->remote_s2p_agent.")";

		techo($conn_msg);
		ctl_broadcast(S2P_PKT_NOTICE, $conn_msg);

		$this->send_pkt_info_srv();

	}
	
	function on_s2p_packet($type, $msg) {

		switch ($type) {

			case S2P_PKT_MASTER_NOTICE:
			techo("* $msg");
			ctl_broadcast(S2P_PKT_MASTER_NOTICE, $msg);
			break;

			case S2P_PKT_MASTER_FATAL:
			techo("FATAL: $msg");
			ctl_broadcast(S2P_PKT_MASTER_FATAL, $msg);
			break;

			case S2P_PKT_PUBMSG:
			$u = sh4_unserialize($msg);
			techo("* ".$u["name"]."> ".$u["msg"]);
			ctl_broadcast(S2P_PKT_PUBMSG, $msg);
			break;

			case S2P_PKT_PRIVMSG:
			$u = sh4_unserialize($msg);
			techo("* [MSG] ".$u["name"]."(".$u["node_id"].")> ".$u["msg"]);
			ctl_broadcast(S2P_PKT_PRIVMSG, $msg);
			break;

			case S2P_PKT_INFO_NET:
			
			$u = sh4_unserialize($msg);

			if (is_array($u["network"])) {
				
				$GLOBALS["net_info"]["network"] = $u["network"];
				$GLOBALS["net_info"]["network"]["network_name"] = ($GLOBALS["master_cnx"] ? $this->server->active_sessions[$GLOBALS["master_cnx"]]->remote_cert_cn : false);

			}

			if (is_array($u["active_nodes"])) $GLOBALS["net_info"]["active_nodes"] = $u["active_nodes"];
			if (is_array($u["nodes"])) $GLOBALS["net_info"]["nodes"] = $u["nodes"];
			
			if (is_array($u["node_update"])) {

				$GLOBALS["net_info"]["active_nodes"][$u["node_update"]["node_id"]] = $u["node_update"];
				
				$GLOBALS["net_info"]["nodes"][$u["node_update"]["node_id"]]["node_short_name"] = $u["node_update"]["node_short_name"];
				$GLOBALS["net_info"]["nodes"][$u["node_update"]["node_id"]]["node_name"] = $u["node_update"]["node_name"];

			}
			
			if ($u["node_disconnect"]) {
				
				unset($GLOBALS["net_info"]["active_nodes"][$u["node_disconnect"]]);

			} else {

				$GLOBALS["update_dlist"] = true;

			}
			
			if (!$GLOBALS["my_node_id"]) foreach ($u["active_nodes"] as $n) if ($GLOBALS["my_cert_cn"] == $n["node_short_name"]) {

				$GLOBALS["my_node_id"] = $n["node_id"];
				break;
			
			}
			
			break;

			case S2P_PKT_RIGHT_LIST:
			$GLOBALS["rl_wait_ctl"]->s2p_write(S2P_PKT_RIGHT_LIST, $msg);
			unset($GLOBALS["rl_wait_ctl"]);
			break;

			case S2P_PKT_FIND:

			$u = sh4_unserialize($msg);
			
			if ($u["req"]) {
			
				// Request
				
				$a = sh4_find_files($u["search"], $GLOBALS["my_search_limit"]);
				$r = sh4_build_result($a);

				$s = sh4_serialize(array("result" => gzdeflate(sh4_serialize($r)), "sid" => $u["sid"]));
				
				$this->s2p_write(S2P_PKT_FIND, $s);

				techo("S ".$u["node_name"]."> ".$u["search"]." (r=".(int)count($a).")");
			
			} else {

				// Reply
				
				$msg = gzinflate($msg);
				
				$GLOBALS["find_wait_ctl"]->s2p_write(S2P_PKT_FIND, $msg);
				unset($GLOBALS["find_wait_ctl"]);

			}

			break;
		
			case S2P_PKT_ASK_HASH:

			$u = sh4_unserialize($msg);

			if ($u["file_hash"]) {

				// Request
			
				if ($fid = sh4_get_file_id_by_hash($u["file_hash"])) {

					$f = sh4_get_file($fid);
					$cfn = $f["file_directory"].DIRECTORY_SEPARATOR.$f["file_name"];

					if (file_exists($cfn)) {

						$opn = true;
						$port = $xfer_key = false;

						// Test for XferMaxUploads
						
						if (($maxup = $GLOBALS["conf"]["xfermaxuploads"][0]) && (tot_up_slots() >= $maxup)) $opn = false;

						// Test for XferMaxUploadsPerNode
						
						if (($maxup = $GLOBALS["conf"]["xfermaxuploadspernode"][0]) && (tot_up_slots($u["node_id"]) >= $maxup)) $opn = false;
						
						if ($opn) {
						
							if (new_xfer_upload_listener($cfn, $GLOBALS["xfer_ports"], $u["ip_addr"], $u["node_id"], $u["node_name"], $port, $xfer_key)) {

								techo("X opening port $port (for ".$u["node_name"].")");
								$ret = true;

							} else {

								techo("! unable to open listening port for file transfer");
								$ret = false;
							
							}
					
						}
					
					} else {

						techo("! unable to read file '$cfn'");
						
						sh4_kill_file($fid);
						$ret = false;
					
					}
				
				} else {

					$ret = false;
				
				}

				if ($ret) {

					$ar = array();

					$ar["aid"] = $u["aid"];
					$ar["ip_port"] = $port;
					$ar["xfer_key"] = $xfer_key;
					$ar["file_name"] = $f["file_name"];
					$ar["file_size"] = $f["file_size"];

					if ($GLOBALS["conf"]["nodepublicaddr"][0]) $ar["ip_addr"] = $GLOBALS["conf"]["nodepublicaddr"][0];

					$as = sh4_serialize($ar);
					$this->s2p_write(S2P_PKT_ASK_HASH, $as);
				
				}
			
			} else {

				// Reply

				if (is_array($GLOBALS["active_downloads"][$adk = $GLOBALS["active_ask"][$u["aid"]]["active_download"]])) {

					$GLOBALS["active_downloads"][$adk]["wait_dlist"][] = $u;

				}

			}
			
			break;


			case S2P_PKT_GET_HASHES:

			$u = sh4_unserialize($msg);
			
			foreach ($u["hashes"] as $hash=>$name) if (!is_active_download($hash)) {
				
				$GLOBALS["active_downloads"][] = array("file_hash" => $hash, "file_name" => $name);

				$msg = "adding '$name' to download queue (by ".$u["node_name"].")";

				techo($msg);
				ctl_broadcast(S2P_PKT_NOTICE, $msg);

			}
			
			break;
		
		}
	
	}

	function on_s2p_fatal($err_msg) {

		techo("! [master] $err_msg");
	
	}

	function on_disconnect() {

		$l_msg = "lost connection to network master server";

		techo($l_msg);
		ctl_broadcast(S2P_PKT_NOTICE, $l_msg);

		unset($GLOBALS["master_cnx"]);
		$GLOBALS["t_nstat"] = 0;
		$GLOBALS["net_info"] = array();

	}

	function send_pkt_info_srv() {

		$srv_info = sh4_server_infos(true);
		$srv_info_s = sh4_serialize($srv_info);
		
		$this->s2p_write(S2P_PKT_INFO_SRV, $srv_info_s);
	
	}

}

// Control connection handler

class ctl_s2p_connection extends s2p_connection {

	var $last_pkt_time;
	var $accept_time = 0;
	
	function on_s2p_accept() {

		$allow = true;
		
		if (is_array($aip = $GLOBALS["conf"]["controlallowip"])) {

			$allow = false;

			foreach ($aip as $as) if (strpos($this->remote_ip, $as) === 0) {

				$allow = true;
				break;
			
			}

		}
		
		if ($allow) {
		
			techo("accepted control connection from ".$this->remote_ip.":".$this->remote_port." (".$this->remote_s2p_agent.")");
	
		} else {

			$this->s2p_write(S2P_PKT_NOTICE, "you are not allowed to connect from this address", false, true);
			techo("denied control connection from ".$this->remote_ip.":".$this->remote_port." (".$this->remote_s2p_agent.")");

		}
	
	}

	function on_s2p_fatal($err_msg) {

		techo("! [ctl] $err_msg");
	
	}
	
	function on_disconnect() {

		if ($this->handshake == 1) {
		
			techo("closed control connection from ".$this->remote_ip.":".$this->remote_port);

		} else {

			techo("? aborted connection from ".$this->remote_ip);
		
		}
		
	}
	
	function on_s2p_packet($type, $msg) {

		switch ($type) {

			case S2P_PKT_INFO_SRV:
			
			$srv_info = sh4_server_infos();
			if (isset($GLOBALS["my_node_id"])) $srv_info["my_node_id"] = $GLOBALS["my_node_id"];
			$srv_info_s = sh4_serialize($srv_info);
			$this->s2p_write(S2P_PKT_INFO_SRV, $srv_info_s);
			
			break;
			
			case S2P_PKT_INFO_NET:
			$ni = array();
			if ($msg & SH4_NI_NETWORK) $ni["network"] = $GLOBALS["net_info"]["network"];
			if ($msg & SH4_NI_ACTIVE_NODES) $ni["active_nodes"] = $GLOBALS["net_info"]["active_nodes"];
			if ($msg & SH4_NI_NODES) $ni["nodes"] = $GLOBALS["net_info"]["nodes"];
			$this->s2p_write(S2P_PKT_INFO_NET, sh4_serialize($ni));
			break;
			
			case S2P_PKT_SHUTDOWN:
			$GLOBALS["exit"] = true;
			$GLOBALS["exit_msg"] = $msg;
			break;
			
			case S2P_PKT_PUBMSG:
			master_write(S2P_PKT_PUBMSG, $msg);
			break;

			case S2P_PKT_PRIVMSG:
			master_write(S2P_PKT_PRIVMSG, $msg);
			break;
		
			case S2P_PKT_RIGHT_LIST:
			$GLOBALS["rl_wait_ctl"] =& $this;
			master_write(S2P_PKT_RIGHT_LIST, $msg);
			break;
		
			case S2P_PKT_RIGHT_SET:
			master_write(S2P_PKT_RIGHT_SET, $msg);
			break;
		
			case S2P_PKT_FIND:

			$u = sh4_unserialize($msg);

			if ($u["local"]) {

				$a = sh4_find_files($u["search"], $GLOBALS["my_search_limit"]);
				$r = sh4_build_result($a, true);
				$s = sh4_serialize($r);

				$this->s2p_write(S2P_PKT_FIND, $s);

				techo("S local: ".$u["search"]." (r=".(int)count($a).")");
			
			} else {

				$f = array("search" => $u["search"], "req" => true);
				$s = sh4_serialize($f);
				
				$GLOBALS["find_wait_ctl"] =& $this;
				master_write(S2P_PKT_FIND, $s);
			
			}

			break;

			case S2P_PKT_GET_HASHES:

			$u = sh4_unserialize($msg);
			
			foreach ($u["hashes"] as $hash=>$name) if (!is_active_download($hash)) {
				
				$GLOBALS["active_downloads"][] = array("file_hash" => $hash, "file_name" => $name);

				$msg = "adding '$name' to download queue";

				techo($msg);
				ctl_broadcast(S2P_PKT_NOTICE, $msg);

			}
			
			break;
			
			case S2P_PKT_CONNECT:
			$GLOBALS["do_master_cnx"] = true;
			$GLOBALS["rc_count"] = 0;
			break;

			case S2P_PKT_DISCONNECT:

			$l_msg = "closed connection to network master server";

			techo($l_msg);
			ctl_broadcast(S2P_PKT_NOTICE, $l_msg);

			if (isset($GLOBALS["master_cnx"])) $GLOBALS["sk"]->active_sessions[$GLOBALS["master_cnx"]]->disconnect();

			$GLOBALS["do_master_cnx"] = false;
			unset($GLOBALS["master_cnx"]);
			$GLOBALS["t_nstat"] = 0;
			$GLOBALS["net_info"] = array();

			break;

			case S2P_PKT_STATUS:
			$st = server_status();
			$this->s2p_write(S2P_PKT_STATUS, sh4_serialize($st));
			break;

			case S2P_PKT_RESCAN:
			
			if ($n_down = count($GLOBALS["active_downloads"])) {
			
				$GLOBALS["update_dlist"] = true;

				$msg = "manual network rescan initiated ($n_down items in download list)";
				
				techo($msg);
				ctl_broadcast(S2P_PKT_NOTICE, $msg);

			}

			break;

			case S2P_PKT_CANCEL:

			if (is_array($GLOBALS["active_downloads"][$msg])) cancel_download($msg);
			
			break;

			case S2P_PKT_KILL:
			if ($u = sh4_unserialize($msg)) kill_sessions($u);
			break;

			case S2P_PKT_PUT_HASHES:
			master_write(S2P_PKT_PUT_HASHES, $msg);
			break;


		}
	
	}

}

// Transfer connection handler

class xfer_s2p_connection extends s2p_connection {

	var $options;

	/*  options array :

		- "remote_node_name"
		- "remote_node_ip";
		- "remote_node_port";
		- "file_hash"

	*/

	var $remote_node_name;
	var $type;
	
	function on_connect_fail() {

		$l_msg = "can't establish xfer connection to ".$this->options["remote_node_name"]." at ".$this->options["remote_node_ip"].":".$this->options["remote_node_port"];
		
		unset($GLOBALS["active_downloads"][$this->options["active_download"]]["active_dlist"][$this->conn_id]);
	
		techo($l_msg);
		ctl_broadcast(S2P_PKT_NOTICE, $l_msg);

		++$GLOBALS["xfer_cnx_errors"];

	}
	
	function on_s2p_accept() {

		$this->xfer_sent_bytes = 0;
		
		++$GLOBALS["xfer_cnx_total"];
		
		if (($GLOBALS["conf"]["xferenforcesourceaddr"][0]) && ($this->remote_ip != $this->options["ip_addr"])) {

			$conn_msg = "rejected xfer connection (waiting for ".$this->options["ip_addr"].", seen ".$this->remote_ip.")";

			++$GLOBALS["xfer_cnx_errors"];

			$this->disconnect();

		} else {
		
			$conn_msg = "accepted xfer connection from ".$this->options["remote_node_name"]." at ".$this->remote_ip.":".$this->remote_port." (".$this->remote_s2p_agent.")";

		}

		techo($conn_msg);
		ctl_broadcast(S2P_PKT_NOTICE, $conn_msg);

		$this->ready = true;

		$GLOBALS["active_listeners"][$this->options["listener_id"]]["accepted"] = true;

		$au = array();
		$au["file_name"] = $this->options["file_name"];
		$au["file_size"] = $this->options["file_size"];
		$au["node_name"] = $this->options["remote_node_name"];
		$au["node_id"] = $this->options["remote_node_id"];
		$au["ip_addr"] = $this->remote_ip;
		$au["ip_port"] = $this->remote_port;
		$au["local_port"] = $this->options["local_port"];
		$au["accept_time"] = time();
		
		$GLOBALS["active_uploads"][$this->conn_id] = $au;

	}
	
	function on_s2p_connect() {

		$this->xfer_rcv_bytes = 0;

		$conn_msg = "established xfer connection to ".$this->options["remote_node_name"]." at ".$this->options["remote_node_ip"].":".$this->options["remote_node_port"]." (".$this->remote_s2p_agent.")";

		techo($conn_msg);
		ctl_broadcast(S2P_PKT_NOTICE, $conn_msg);

		$GLOBALS["active_downloads"][$this->options["active_download"]]["active_dlist"][$this->conn_id]["connect_time"] = time();

		$this->xfer_status = 0;

		++$GLOBALS["xfer_cnx_total"];

	}
	
	function on_s2p_packet($type, $msg) {

		switch ($type) {

			case S2P_PKT_DATA:

			if ($this->ctype == "client") {

				$wb = array();

				$wb["block"] = $this->req_block_id;
				$wb["data"] = $msg;
				
				$GLOBALS["active_downloads"][$this->options["active_download"]]["wait_blocks"][] = $wb;

				$this->xfer_status = 0;
				
				$xfer_len = strlen($msg);
				
				$this->xfer_rcv_bytes += $xfer_len;
				$GLOBALS["xfer_rcv_bytes"] += $xfer_len;
			
			} else {

				$u = sh4_unserialize($msg);

				if ($f = @fopen($this->options["file_name"], "r")) {

					$length = ($u["end"] - $u["start"]) + 1;

					fseek($f, $u["start"]);
					$data = fread($f, $length);
					fclose($f);

					$this->s2p_write(S2P_PKT_DATA, $data);

					$this->xfer_sent_bytes += $length;
					$GLOBALS["xfer_sent_bytes"] += $length;

					$GLOBALS["active_uploads"][$this->conn_id]["block_start"] = $u["start"];
					$GLOBALS["active_uploads"][$this->conn_id]["block_end"] = $u["end"];

				} else {

					techo("! unable to read file '".$this->options["file_name"]."'");
				
				}
			
			}

			break;

		}

	}

	function on_s2p_fatal($err_msg) {

		techo("! [xfer] $err_msg");
	
	}

	function on_disconnect() {

		if ($this->ctype == "client") {

			$l_msg = "xfer connection dropped by ".$this->options["remote_node_name"]." at ".$this->options["remote_node_ip"].":".$this->options["remote_node_port"]." (rcvd ".sh4_size_format($this->xfer_rcv_bytes).")";
			
			$GLOBALS["active_downloads"][$this->options["active_download"]]["file_blocks"]{$this->req_block_id} = SH4_BLK_FREE;
			unset($GLOBALS["active_downloads"][$this->options["active_download"]]["active_dlist"][$this->conn_id]);
		
			techo($l_msg);
			ctl_broadcast(S2P_PKT_NOTICE, $l_msg);

		} else if ($this->ctype == "server") {

			$l_msg = "closed xfer connection from ".$this->options["remote_node_name"]." at ".$this->remote_ip.":".$this->remote_port." (sent ".sh4_size_format($this->xfer_sent_bytes).")";
			
			unset($GLOBALS["active_uploads"][$this->conn_id]);

			techo($l_msg);
			ctl_broadcast(S2P_PKT_NOTICE, $l_msg);
		
		}

	}

}


// Other funcs

function sh4_server_infos($update_cache=false) {

	static $ret_cache;
	
	if ($update_cache || !is_array($ret_cache)) {
	
		$ret = array();

		$ret["node_name"] = trim($GLOBALS["conf"]["nodename"][0]);
		$ret["node_url"] = trim($GLOBALS["conf"]["nodeurl"][0]);
		$ret["node_motd"] = @file_get_contents($GLOBALS["conf"]["nodemotdfile"][0]);

		$ret["node_files_a"] = $ret["node_mbytes_a"] = $ret["node_files_v"] = $ret["node_mbytes_v"] = $ret["node_files_o"] = $ret["node_mbytes_o"] = 0;
		
		$q = mysql_query("select file_type, count(*), sum(file_size)/1048576 from files group by file_type");

		while ($r = mysql_fetch_row($q)) switch ($r[0]) {

			case "Audio":
			$ret["node_files_a"] = $r[1];
			$ret["node_mbytes_a"] = $r[2];
			break;

			case "Video":
			$ret["node_files_v"] = $r[1];
			$ret["node_mbytes_v"] = $r[2];
			break;

			case "Other":
			$ret["node_files_o"] = $r[1];
			$ret["node_mbytes_o"] = $r[2];
			break;
		
		}

		mysql_free_result($q);
		
		$ret_cache = $ret;
	
	} else {

		$ret = $ret_cache;
	
	}
	
	return $ret;

}

function server_status() {
	
	$st = array();

	$st["current_time"] = time();
	$st["start_time"] = $GLOBALS["start_time"];
	$st["master_cnx"] = $GLOBALS["master_cnx"];

	$ups_s = $GLOBALS["conf"]["xfermaxuploads"][0]
	or $ups_s = "-";
	
	$st["slots_up"] = $ups_s;
	$st["slots_up_used"] = tot_up_slots();

	$st["xfer_cnx_total"] = $GLOBALS["xfer_cnx_total"];
	$st["xfer_cnx_errors"] = $GLOBALS["xfer_cnx_errors"];
	$st["xfer_sent_bytes"] = $GLOBALS["xfer_sent_bytes"];
	$st["xfer_rcv_bytes"] = $GLOBALS["xfer_rcv_bytes"];
	
	$st["downloads"] = $st["uploads"] = $st["control"] = array();
	
	foreach ($GLOBALS["sk"]->active_sessions as $k=>$as) if (is_a($as, "ctl_s2p_connection")) {

		$ctl = array();

		$ctl["remote_ip"] = $as->remote_ip;
		$ctl["remote_port"] = $as->remote_port;
		$ctl["accept_time"] = $as->accept_time;
		$ctl["idle_time"] = ($as->last_read_time ? (time() - $as->last_read_time) : 0);

		$st["control"][$as->conn_id] = $ctl;
		
	}

	foreach ($GLOBALS["active_downloads"] as $k=>$ad) {

		$dl = array();

		$dl["file_name"] = $ad["file_name"];
		$dl["file_size"] = $ad["file_size"];

		$adl = array();

		if ($ad["active_dlist"]) foreach ($ad["active_dlist"] as $adlk=>$adlv) {

			$adl[$adlk] = $adlv;
			$adl[$adlk]["rate_in"] = $GLOBALS["sk"]->active_sessions[$adlk]->statistics["rate_in"];
			$adl[$adlk]["rate_out"] = $GLOBALS["sk"]->active_sessions[$adlk]->statistics["rate_out"];
		
		}
		
		$dl["connections"] = $adl;

		$tot_blocks = $cur_blocks = $dl_blocks = 0;
		
		if ($ad["file_blocks"]) {
			
			$tot_blocks = strlen($ad["file_blocks"]);
			
			for ($a = 0; $a < $tot_blocks; $a++) switch ($ad["file_blocks"]{$a}) {

				case SH4_BLK_CUR: ++$cur_blocks;
				case SH4_BLK_DONE: ++$dl_blocks;
			
			}
			
			$dl["total_blocks"] = $tot_blocks;
			$dl["current_blocks"] = $cur_blocks;
			$dl["dl_blocks"] = $dl_blocks;
			
			$bl_s = "";
			$bmul = ($tot_blocks - 1) / 25;

			for ($a = 0; $a < 25; $a++) {
				
				switch ($ad["file_blocks"]{(int)($a*$bmul)}) {

					case SH4_BLK_FREE: 
					case SH4_BLK_CUR: 
					$bl_c = "-";
					break;

					case SH4_BLK_DONE:
					$bl_c = "#";
			
				}

				$bmin = (int)($a * $bmul);
				$bmax = (int)(($a + 1) * $bmul);
				
				for ($b = $bmin; $b <= $bmax; $b++) if ($ad["file_blocks"]{$b} == SH4_BLK_CUR) {
					
					$bl_c = "|";
					break;

				}

				$bl_s .= $bl_c;

			}
			
			$dl["blocks_25c"] = $bl_s;

		}
		
		$st["downloads"][$k] = $dl;
	
	}

	foreach ($GLOBALS["active_uploads"] as $k=>$au) {

		$ul = array();

		$ul = $au;

		$bl_s = "-------------------------";
		$bmul = $au["file_size"] / 24;
		if ($bmul < 1) $bmul = 1;
		$bstep = $bmul / 3;

		for ($a = $au["block_start"]; $a <= $au["block_end"]; $a += $bstep) {

			$bl_s{(int)$a/$bmul} = "|";
		
		}

		$ul["rate_in"] = $GLOBALS["sk"]->active_sessions[$k]->statistics["rate_in"];
		$ul["rate_out"] = $GLOBALS["sk"]->active_sessions[$k]->statistics["rate_out"];
		
		$ul["blocks_25c"] = $bl_s;
		$st["uploads"][$k] = $ul;
	
	}

	return $st;

}

function sigusr1_handler() {

	$u_msg = "SIGUSR1 received, reloading catalog summary from database";

	techo($u_msg);
	ctl_broadcast(S2P_PKT_NOTICE, $u_msg);

	if ($m = $GLOBALS["sk"]->active_sessions[$GLOBALS["master_cnx"]]) $m->send_pkt_info_srv();

}

function ctl_broadcast($type, $msg) {

	foreach ($GLOBALS["sk"]->active_sessions as $k=>$as) if (is_a($as, "ctl_s2p_connection")) $GLOBALS["sk"]->active_sessions[$k]->s2p_write($type, $msg);

}

function master_write($type, $msg) {

	if (isset($GLOBALS["sk"]->active_sessions[$GLOBALS["master_cnx"]])) $GLOBALS["sk"]->active_sessions[$GLOBALS["master_cnx"]]->s2p_write($type, $msg);

}

function is_active_download($hash) {

	foreach ($GLOBALS["active_downloads"] as $ad) if ($ad["file_hash"] == $hash) return true;
	return false;

}

function new_xfer_upload_listener($file_name, $port_range, $ip_addr, $remote_node_id, $remote_node_name, &$port, &$net_key) {

	$net_key = md5($GLOBALS["net_key"].$file_name.$ip_addr.fileinode($file_name).time());

	$nb_ports = ($port_range[1] - $port_range[0]) + 1;
	$x = false;
	$port_try = 0;

	while ((!$x) && ($port_try++ <= $nb_ports)) {
	
		$port = mt_rand($port_range[0], $port_range[1]);
		$lsn_addr = "tcp://0.0.0.0:$port";
		
		$sk_options = array(	"s2p_agent"			=> "sh4red/".SH4_VERSION,
								"net_key"			=> $net_key,
								"file_name"			=> $file_name,
								"file_size"			=> filesize($file_name),
								"ip_addr"			=> $ip_addr,
								"local_port"		=> $port,
								"listener_id"		=> $lsn_addr,
								"remote_node_id"	=> $remote_node_id,
								"remote_node_name"	=> $remote_node_name);

		
		$x = $GLOBALS["sk"]->register_connection_handler($lsn_addr, "xfer_s2p_connection", 5, $sk_options);

	}

	if ($x) {
	
		$al = array();

		$al["pap"] = $lsn_addr;
		$al["ip_port"] = $port;
		$al["handler_id"] = $x;
		$al["node_id"] = $remote_node_id;
		$al["start"] = time();

		$GLOBALS["active_listeners"][$lsn_addr] = $al;
	
	}
	
	return $x;

}

function xfer_first_free_block($file_blocks) {

	$ret = strpos($file_blocks, SH4_BLK_FREE);

	return $ret;

}

function xfer_download_complete($file_blocks) {

	
	$ret = ((strpos($file_blocks, SH4_BLK_FREE) === false) && (strpos($file_blocks, SH4_BLK_CUR) === false));
	
	return $ret;

}

function file_block_info($block_id, $file_size) {

	$st = $bl["start"] = $block_id*$GLOBALS["xfer_block_size"];
	$e = (($st += $GLOBALS["xfer_block_size"]) > $file_size) ? $file_size : $st;
	$bl["end"] = $e - 1;

	return $bl;

}

function write_dot_sh4($ad = false) {

	if ($ad === false) {

		foreach ($GLOBALS["active_downloads"] as $ad) if ($ad["file_name"]) write_dot_sh4($ad);
		return true;

	} else {

		if ($ad["file_name"]) {

			$dsn = $GLOBALS["idl_dir"].DIRECTORY_SEPARATOR.$ad["file_name"].".sh4";

			if (!$f = @fopen($dsn, "w")) {

				techo("! unable to write to '$dns'");
				return false;
			
			}

			$dd = array();

			$dd["file_name"] = $ad["file_name"];
			$dd["file_size"] = $ad["file_size"];
			$dd["file_hash"] = $ad["file_hash"];
			$dd["asked_dlist"] = $ad["asked_dlist"];
			$dd["file_blocks"] = str_replace(SH4_BLK_CUR, SH4_BLK_FREE, $ad["file_blocks"]);
			
			$s = sh4_serialize($dd);
			$s = gzdeflate($s);
			fwrite($f, $s);
			fclose($f);
		
		}

	}

}

function unlink_dot_sh4($ad) {

	$r = @unlink($GLOBALS["idl_dir"].DIRECTORY_SEPARATOR.$ad["file_name"].".sh4");

	return $r;

}

function load_dot_sh4($filename = false) {

	if ($filename === false) {

		if (!$d = @opendir($GLOBALS["idl_dir"])) return false;

		while (($f = readdir($d)) !== false) {

			$res = array();

			if (ereg("^(.*)\.sh4$", $f, $res)) {

				$dfn = $res[1];
				$qfn = $res[1].".sh4";
				
				if (file_exists($GLOBALS["idl_dir"].DIRECTORY_SEPARATOR.$dfn)) {
				
					if ($ad = load_dot_sh4($qfn)) {
						
						$GLOBALS["active_downloads"][] = $ad;
						$r++;
						
					} else {
						
						techo("? invalid queue file '$qfn'");

					}
		
				} else {

					if (strpos($ad["file_blocks"], SH4_BLK_DONE) !== false) {
					
						techo("? can not find '$dfn', please delete '$qfn' if it's no longer valid");

					}
				
				}
			
			}
		
		}
		
		closedir($d);
	
	} else {
	
		if (!$s = @file_get_contents($GLOBALS["idl_dir"].DIRECTORY_SEPARATOR.$filename)) return false;
		if (!$s = @gzinflate($s)) return false;
		if (!$r = sh4_unserialize($s)) return false;

	}

	return $r;

}

function tot_up_slots($node_id = false) {

	if ($node_id === false) {

		$ret = count($GLOBALS["active_uploads"]) + count($GLOBALS["active_listeners"]);
	
	} else {

		$ret = 0;
		
		foreach ($GLOBALS["active_uploads"] as $au) if ($au["node_id"] == $node_id) ++$ret;
		foreach ($GLOBALS["active_listeners"] as $al) if ($al["node_id"] == $node_id) ++$ret;

	}

	return $ret;

}

function cancel_download($ad_key) {

	$drp_c = 0;
	
	if (!is_array($GLOBALS["active_downloads"][$ad_key])) return false;

	$ad = $GLOBALS["active_downloads"][$ad_key];

	if ($ad["active_dlist"]) foreach ($ad["active_dlist"] as $adk=>$adl) {
		
		if (is_object($GLOBALS["sk"]->active_sessions[$adl["conn_id"]])) {
			
			$GLOBALS["sk"]->active_sessions[$adl["conn_id"]]->disconnect();
			++$drp_c;

		}

	}

	if (file_exists($cfn = ($GLOBALS["idl_dir"].DIRECTORY_SEPARATOR.$ad["file_name"]))) unlink($cfn);
	if (file_exists($cfn = ($GLOBALS["idl_dir"].DIRECTORY_SEPARATOR.$ad["file_name"].".sh4"))) unlink($cfn);

	$c_msg = "canceled download of '".$ad["file_name"]."' (dropped $drp_c connections)";

	techo("* ".$c_msg);
	ctl_broadcast(S2P_PKT_NOTICE, $c_msg);
	
	unset($GLOBALS["active_downloads"][$ad_key]);

}

function kill_sessions($sess_ids, $reason="requested") {

	if ($sess_ids) foreach ($sess_ids as $sess_id) if (is_object($GLOBALS["sk"]->active_sessions[$sess_id])) {
	
		if (is_a($GLOBALS["sk"]->active_sessions[$sess_id], "xfer_s2p_connection")) {

			$xb = "(".(($GLOBALS["sk"]->active_sessions[$sess_id]->ctype == "client") ? ("rcvd ".sh4_size_format($GLOBALS["sk"]->active_sessions[$sess_id]->xfer_rcv_bytes)) : ("sent ".sh4_size_format($GLOBALS["sk"]->active_sessions[$sess_id]->xfer_sent_bytes))).") ";
		
		}
		
		$k_msg = "killed ".($xb ? "xfer" : "control")." connection ".$sess_id." ".$xb.": ".$reason;

		if (is_array($ad = $GLOBALS["active_downloads"][$GLOBALS["sk"]->active_sessions[$sess_id]->options["active_download"]])) {
			
			if (isset($GLOBALS["sk"]->active_sessions[$sess_id]->req_block_id))	
			$GLOBALS["active_downloads"][$GLOBALS["sk"]->active_sessions[$sess_id]->options["active_download"]]["file_blocks"]{$GLOBALS["sk"]->active_sessions[$sess_id]->req_block_id} = SH4_BLK_FREE;
		
			$ad = $GLOBALS["active_downloads"][$GLOBALS["sk"]->active_sessions[$sess_id]->options["active_download"]];
						
			write_dot_sh4($ad);
		
		}
		
		if (is_a($GLOBALS["sk"]->active_sessions[$sess_id], "xfer_s2p_connection")) {

			if (is_array($GLOBALS["active_uploads"][$sess_id])) {

				// upload
			
				unset($GLOBALS["active_uploads"][$sess_id]);
			
			} else {

				// download
			
				unset($GLOBALS["active_downloads"][$GLOBALS["sk"]->active_sessions[$sess_id]->options["active_download"]]["active_dlist"][$sess_id]);

			}
		
		}
		
		$GLOBALS["sk"]->active_sessions[$sess_id]->disconnect();
		
		techo("* ".$k_msg);
		ctl_broadcast(S2P_PKT_NOTICE, $k_msg);

	}

}

// Init

$start_time = time();

$active_ask = array();
$active_downloads = array();
$active_uploads = array();
$active_listeners = array();

$xfer_sent_bytes = $xfer_rcv_bytes = $xfer_cnx_total = $xfer_cnx_errors = 0;

$cmdline = sh4_parse_cmdline_args(array(	"-h" => "--help",
											"-V" => "--version",
											"-n" => "--new-network-cert",
											"-d" => "--start-daemon"));

if ($cmdline["-V"]) die(SH4_VERSION."\n");
if ($cmdline["-h"]) die(SH4RED_HELP);

techo("sh4red/".SH4_VERSION." (c) 2003 by sIX / aEGiS <six@t0x.net>");

pcntl_signal(SIGUSR1, "sigusr1_handler");

if (version_compare(phpversion(), "4.3.0")<0) errexit("FATAL: sh4re-update needs PHP >= 4.3)");
if (!is_callable("openssl_x509_read")) errexit("FATAL: openssl extension not available");

$conf_f = $cmdline["-c"]
or $conf_f = SH4_DEFAULT_CONF_FILE;

if (!$conf = sh4_parse_config($conf_f)) errexit("FATAL: unable to read conf file '$conf_f'");

$xfer_lsn_timeout = $conf["xferlistenertimeout"][0]
or $xfer_lsn_timeout = SH4_DEFAULT_XFER_LISTENER_TIMEOUT;

$dlist_ask_interval = $conf["availscaninterval"][0]
or $dlist_ask_interval = SH4_DEFAULT_DLIST_ASK_INTERVAL;

$hash_ask_timeout = $conf["availscantimeout"][0]
or $hash_ask_timeout = SH4_DEFAULT_ASK_TIMEOUT;

$my_ca = array($conf["sslcert"][0]);
$my_cert_s = @file_get_contents($conf["sslcert"][0]);
$my_cert = @openssl_x509_read($my_cert_s);
$my_cert_p = @openssl_x509_parse($my_cert);
$my_cert_cn = $my_cert_p["subject"]["CN"];
@openssl_x509_export($my_cert, $my_cert_s);
$my_key_s = @file_get_contents($conf["sslkey"][0]);
$my_public_key = @openssl_pkey_get_public($my_cert_s);
$my_key = @openssl_pkey_get_private($my_key_s);
$net_key = trim($conf["networkkey"][0]);
$ctl_key = trim($conf["controlkey"][0]);
$pid_file = trim($conf["pidfile"][0]);
$xfer_ports = explode("-", $conf["xferports"][0]);
$dl_dir = trim($conf["downloaddir"][0]);
$idl_dir = trim($conf["workingdir"][0]);

unset($my_key_s);

if (!is_dir($dl_dir)) errexit("FATAL: download dir '$dl_dir' does not exist");
if (!is_dir($idl_dir)) errexit("FATAL: working dir '$idl_dir' does not exist");


$lsn_ctl = trim($conf["controllistener"][0]);
$lsn_s = "tcp://".$lsn_ctl;

if (!$my_public_key) errexit("FATAL: unable to load SSL certificate from ".$conf["sslcert"][0]);
if (!$my_key) errexit("FATAL: unable to load SSL private key from ".$conf["sslkey"][0]);
if (!ssl_check_keys($my_key, $my_public_key)) errexit("FATAL: crypto test failed, certificate and private key don't match");

techo("SSL keys loaded, crypto test passed");

if ((!mysql_connect($conf["mysqlhost"][0], $conf["mysqluser"][0], $conf["mysqlpass"][0])) || (!mysql_select_db($conf["mysqldb"][0]))) errexit("FATAL: unable to connect to MySQL");

techo("connected to MySQL database");

if ($n_reloaded = load_dot_sh4()) techo("reloaded download queue (".(int)$n_reloaded." items)");

if (!$master_s = trim($conf["networkmaster"][0])) errexit("FATAL: can't find master address in config file");

$master_host = strtok($master_s, ":");
$master_port = strtok("");

if (!$master_port) $master_port = SH4_DEFAULT_MASTER_PORT;

$master_s = "tcp://$master_host:$master_port";

// netlib init

$sk = new sk_server;
techo($sk->sk_info());

$sk->set_option("statistics", true);
$sk->set_option("reuse_address", true);

// setup listening control connection

$sk_c_options = array(	"s2p_agent"				=> "sh4red/".SH4_VERSION,
						"net_key"				=> $ctl_key);

if (!$sk->register_connection_handler($lsn_s, "ctl_s2p_connection", 20, $sk_c_options)) errexit("FATAL: listen failed (".$sk->error.")");
techo("listening for control connections at $lsn_s");

// setup outgoing master connetion

techo("connecting to network master server at $master_s");

$sk_m_options = array(	"s2p_agent"				=> "sh4red/".SH4_VERSION,
						"net_key"				=> $net_key,
						"use_rsa"				=> true,
						"my_key"				=> $my_key,
						"my_cert_s"				=> $my_cert_s,
						"check_network_cert"	=> true,
						"new_network_cert"		=> (isset($cmdline["-n"])),
						"network_cert_file"		=> $conf["networkcert"][0]);

if (!$master_cnx = $sk->register_outgoing_connection($master_s, "masterd_s2p_connection", $sk_m_options, 5, true)) errexit("FATAL: unable to contact network master (".$sk->error.")");

unset($sk_m_options["new_network_cert"]);

$net_info = array();
$rc_count = $rc_wait = 0;

if ($cmdline["-d"]) {

	if (SK_Toolkit::Run_In_Background()) {

		techo("started daemon mode, running in background");

	} else {

		techo("unable to start daemon mode");
	
	}

}

if ($f = @fopen($pid_file, "w")) {

	fputs($f, getmypid()."\n");
	fclose($f);

} else {

	techo("can't open pid file '$pid_file' for writing");

}

$my_search_limit = $conf["searchlimit"][0]
or $my_search_limit = SH4_DEFAULT_SEARCH_LIMIT;

$xfer_block_size = $conf["xferblocksize"][0]
or $xfer_block_size = SH4_DEFAULT_XFER_BLOCK_SIZE;

// main loop

$do_master_cnx = true;

while (!$exit) {

	declare (ticks = 1) {

		// Do nothing but catch signals
	
	}
	
	$sk->run(1);

	$t = time();
	
	if (!$master_cnx && $do_master_cnx) {
		
		// Connection to master lost !
		
		if ($rc_wait++ >= ($rc_count-1)) {
		
			++$rc_count;

			if (!$master_cnx = $sk->register_outgoing_connection($master_s, "masterd_s2p_connection", $sk_m_options, 5, true)) {
				
				$l_msg = "master reconnect failed (".$sk->error.") retrying in ".$rc_count." sec";

				techo($l_msg);
				ctl_broadcast(S2P_PKT_NOTICE, $l_msg);

			} else {

				$rc_count = 0;
			
			}

			$rc_wait = 0;

		}

	}
	
	// Handle active asks

	foreach ($active_ask as $k=>$aa) {

		if ($aa["start"] + $hash_ask_timeout <= $t) {
			
			unset($active_ask[$k]);

		}
	
	}
	
	// Handle listeners

	foreach ($active_listeners as $k=>$al) {

		if (($al["start"] + $xfer_lsn_timeout <= $t) || ($al["accepted"])) {
			
			techo("X closing port ".$al["ip_port"]." (".($al["accepted"]?"accepted":"timeout").")");
			$sk->unregister_connection_handler($al["pap"]);
			unset($active_listeners[$k]);

		}
	
	}
	
	// Handle file downloads

	foreach ($active_downloads as $k=>$ad) {
	
		if ($update_dlist) {
			
			$ad["asked_dlist"] = 0;
			$update_dlist = false;

		}
		
		if ($ad["asked_dlist"] + $dlist_ask_interval <= $t) {

			$ah = array();
			$aid++;

			$ah["aid"] = $aid;
			$ah["file_hash"] = $ad["file_hash"];
			$ah["exclude_nodes"] = array($GLOBALS["my_node_id"]);
			if ($ad["active_dlist"]) foreach ($ad["active_dlist"] as $adl) $ah["exclude_nodes"][] = $adl["node_id"];

			if ($GLOBALS["conf"]["nodepublicaddr"][0]) $ah["ip_addr"] = $GLOBALS["conf"]["nodepublicaddr"][0];
			
			master_write(S2P_PKT_ASK_HASH, sh4_serialize($ah));
			$active_downloads[$k]["asked_dlist"] = time();

			$active_ask[$aid] = array("file_hash" => $ad["file_hash"], "active_download" => $k, "start" => time());

		}

		// Connect to waiting download list
		
		$ad = $active_downloads[$k];
		
		if ($ad["wait_dlist"]) foreach ($ad["wait_dlist"] as $wdk=>$wc) {

			if (!$ad["file_size"]) $active_downloads[$k]["file_size"] = $wc["file_size"];
			if (!$ad["file_name"]) $active_downloads[$k]["file_name"] = $wc["file_name"];
			
			$already_connected = false;
			if ($ad["active_dlist"]) foreach ($ad["active_dlist"] as $adl) if ($wc["node_id"] == $adl["node_id"]) $already_connected = true;
			
			if (!$already_connected) {
			
				$pap = "tcp://".$wc["ip_addr"].":".$wc["ip_port"];

				$xf_options = array(	"s2p_agent"			=> "sh4red/".SH4_VERSION,
										"net_key"			=> $wc["xfer_key"],
										"remote_node_name"	=> $wc["node_name"],
										"remote_node_ip"	=> $wc["ip_addr"],
										"remote_node_port"	=> $wc["ip_port"],
										"active_download"	=> $k,
										"my_pap"			=> $pap);

				if ($c = $sk->register_outgoing_connection($pap, "xfer_s2p_connection", $xf_options)) {

					$ad = array();

					$ad["node_id"] = $wc["node_id"];
					$ad["node_name"] = $wc["node_name"];
					$ad["ip_addr"] = $wc["ip_addr"];
					$ad["ip_port"] = $wc["ip_port"];
					$ad["conn_id"] = $c;
					$ad["status"] = 0;

					$active_downloads[$k]["active_dlist"][$c] = $ad;

				} else {

					techo("X unable to open xfer connection to ".$wc["node_name"]." at $pap");

				}

			}
			
			unset($active_downloads[$k]["wait_dlist"][$wdk]);
		
		}

		$ad = $active_downloads[$k];
		
		// Build blocks array

		if ((!$ad["file_blocks"]) && (isset($ad["file_size"]))) {

			$fb = "";

			$a = 0;
			
			do { 
				
				$a += $GLOBALS["xfer_block_size"];
				$fb .= SH4_BLK_FREE;

			} while ($a < $ad["file_size"]);
			
			$active_downloads[$k]["file_blocks"] = $fb;

			$ad = $active_downloads[$k];
			write_dot_sh4($ad);

		}

		// Manage active connections for this file
				
		if ($ad["active_dlist"]) foreach ($ad["active_dlist"] as $adk=>$adl) {
		
			switch ($adl["status"]) {

				case 0: // TCP connection done, waiting for s2p handshake
				if ($sk->active_sessions[$adl["conn_id"]]->handshake == 1) $active_downloads[$k]["active_dlist"][$adk]["status"] = 1;
				break;

				case 1: // s2p connection up

				if ((!$ad["file_blocks"]) || ($sk->active_sessions[$adl["conn_id"]]->xfer_status != 0)) break;
				
				$nb = xfer_first_free_block($ad["file_blocks"]);

				if ($nb === false) break;

				$active_downloads[$k]["file_blocks"]{$nb} = SH4_BLK_CUR;

				$ar = file_block_info($nb, $ad["file_size"]);
				$s = sh4_serialize($ar);

				if (!is_object($sk->active_sessions[$adl["conn_id"]])) break;

				$sk->active_sessions[$adl["conn_id"]]->req_block_id = $nb;
				$sk->active_sessions[$adl["conn_id"]]->xfer_status = 1;
				$sk->active_sessions[$adl["conn_id"]]->s2p_write(S2P_PKT_DATA, $s);
				
				break;
			
			}
			
			$ad = $active_downloads[$k];
		
		}
	
		// Build dest file if we have waiting blocks

		if ($ad["wait_blocks"]) {
			
			$cfn = $idl_dir.DIRECTORY_SEPARATOR.$ad["file_name"];
			
			if (!file_exists($cfn)) {

				if ($f = @fopen($cfn, "w")) {

					fclose($f);

				} else {

					techo("! unable to create file for writing : ".$cfn);
				
				}
			
			}
			
			foreach ($ad["wait_blocks"] as $wbk=>$wb) {

				// write it !
				
				if ($f = @fopen($cfn, "r+")) {

					$ar = file_block_info($wb["block"], $ad["file_size"]);
					
					fseek($f, $ar["start"]);
					fwrite($f, $wb["data"]);										
					fclose($f);

				} else {

					techo("! unable to write to file : ".$cfn);
					break;
				
				}
				
				$active_downloads[$k]["file_blocks"]{$wb["block"]} = SH4_BLK_DONE;
				unset($active_downloads[$k]["wait_blocks"][$wbk]);

			}

			$ad = $active_downloads[$k];
			write_dot_sh4($ad);
			
			if (xfer_download_complete($ad["file_blocks"])) {

				$d_msg = "download of '".$ad["file_name"]."' complete";

				techo($d_msg);
				ctl_broadcast(S2P_PKT_NOTICE, $d_msg);
				
				// verify file - move file

				$h = sh4_compute_hash($cfn);

				if ($h != $ad["file_hash"]) {

					$d_msg = "WARN: '".$ad["file_name"]."' verfication failed !";

					techo($d_msg);
					ctl_broadcast(S2P_PKT_NOTICE, $d_msg);
				
				} else {

					if (!rename($cfn, $dl_dir.DIRECTORY_SEPARATOR.$ad["file_name"])) {

						$d_msg = "WARN: '".$ad["file_name"]."' move to download dir failed !";

						techo($d_msg);
						ctl_broadcast(S2P_PKT_NOTICE, $d_msg);

					} else {

						$d_msg = "downloaded file '".$ad["file_name"]."' verified and moved to download dir";

						techo($d_msg);
						ctl_broadcast(S2P_PKT_NOTICE, $d_msg);
					
					}

				}
				
				if ($ad["active_dlist"]) foreach ($ad["active_dlist"] as $adl) {
					
					$x_msg = "closed xfer connection to ".$sk->active_sessions[$adl["conn_id"]]->options["remote_node_name"]." at ".$sk->active_sessions[$adl["conn_id"]]->options["remote_node_ip"].":".$sk->active_sessions[$adl["conn_id"]]->options["remote_node_port"]." (rcvd ".sh4_size_format($sk->active_sessions[$adl["conn_id"]]->xfer_rcv_bytes).")";

					techo($x_msg);
					ctl_broadcast(S2P_PKT_NOTICE, $x_msg);

					$sk->active_sessions[$adl["conn_id"]]->disconnect();

				}
				
				unlink_dot_sh4($ad);
				unset($active_downloads[$k]);
				$update_dlist = true;
			
			}
		
		}
	
	}

	foreach ($sk->active_sessions as $k=>$as) if ($as->s2p_errors) {
		
		foreach ($as->s2p_errors as $err) techo("[".$as->remote_ip.":".$as->remote_port."] $err");
		unset($sk->active_sessions[$k]->s2p_errors);

	}

	if ((($t_nstat + 3600) <= $t) && ($net_info)) {

		$nb_nodes = count($net_info["active_nodes"]);
		
		$ni_msg = $nb_nodes." node".($nb_nodes>1?"s":"")." online, ".sh4_size_format($net_info["network"]["mbytes_total"]*1048576)." available in ".$net_info["network"]["files_total"]." files";

		techo("* ".$ni_msg);
		ctl_broadcast(S2P_PKT_NOTICE, $ni_msg);

		$t_nstat = $t;
	
	}

	// Collect bitrate statistics, disconnect timed out xfers

	$mt = get_microtime();
	
	foreach ($sk->active_sessions as $ask=>$as) {

		if (!$as->statistics) {

			$sk->active_sessions[$ask]->statistics = array("time" => $mt, "bytes_out" => 0, "bytes_in" => 0, "rate_out" => 0, "rate_in" => 0);
			$sk->active_sessions[$ask]->statistics_history = array();
			$as = $sk->active_sessions[$ask];
		
		}
		
		if (($as->statistics["time"] + SH4_RATE_SAMPLE_INTERVAL) <= $mt) {

			// time to update

			$stats = array();
			$stats["time"] = $mt;
			$stats["bytes_out"] = $as->socket->statistics["bytes_out"];
			$stats["bytes_in"] = $as->socket->statistics["bytes_in"];

			array_push($sk->active_sessions[$ask]->statistics_history, $stats);

			$st_first = false;
			
			foreach ($sk->active_sessions[$ask]->statistics_history as $hst) {

				if ($st_first === false) $st_first = $hst;

				if (($hst["time"] + SH4_RATE_AVG_INTERVAL) <= $mt) {

					array_shift($sk->active_sessions[$ask]->statistics_history);
				
				}
			
			}
			
			$t_diff = $mt - $st_first["time"];
			if ($t_diff == 0) $t_diff = 1;
			
			$stats["rate_out"] = (int)(($stats["bytes_out"] - $st_first["bytes_out"]) / $t_diff);
			$stats["rate_in"] = (int)(($stats["bytes_in"] - $st_first["bytes_in"]) / $t_diff);

			$sk->active_sessions[$ask]->statistics = $stats;

		}
	
		$stalled_xfers = array();
		
		if ((is_a($as, "xfer_s2p_connection") && ($as->handshake) && ($to = $conf["xferstalledtimeout"][0]) && (($as->last_read_time + $to) < $t))) $stalled_xfers[] = $ask;

		if ($stalled_xfers) kill_sessions($stalled_xfers, "stalled");
	
	}

	// Send noop packets every minute to keep connection alive

	if (($t_noop + 60) <= $t) {

		foreach ($sk->active_sessions as $ask=>$as) if ($as->handshake) $sk->active_sessions[$ask]->s2p_write(S2P_PKT_WOOPWOOPWOOP);
		$t_noop = $t;

	}

}

techo("sh4red stopped".($exit_msg?" ($exit_msg)":"")."\n");

if ($pid_file) unlink($pid_file);

?>
